@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: proc.ml,v 1.4 1999/11/17 18:56:40 xleroy Exp $ *)
+(* $Id: proc.ml,v 1.6 2002/07/22 16:37:46 doligez Exp $ *)
 
 (* Description of the Alpha processor *)
 
@@ -211,5 +211,7 @@ let assemble_file infile outfile =
     then if !Clflags.gprofile then "as -O2 -nocpp -pg -o "
                               else "as -O2 -nocpp -o "
     else "as -o " in
-  Ccomp.command (as_cmd ^ outfile ^ " " ^ infile)
+  Ccomp.command (as_cmd ^ Filename.quote outfile ^ " " ^ Filename.quote infile)
 
+open Clflags;;
+open Config;;
