@@ -10,13 +10,17 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: arch.ml,v 1.8 2000/04/21 08:10:54 weis Exp $ *)
+(* $Id: arch.ml,v 1.9 2002/11/29 15:03:36 xleroy Exp $ *)
 
 (* Specific operations for the ARM processor *)
 
 open Misc
 open Format
 
+(* Machine-specific command-line options *)
+
+let command_line_options = []
+
 (* Addressing modes *)
 
 type addressing_mode =
