@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: asmlink.ml,v 1.57 2002/07/22 08:07:26 xleroy Exp $ *)
+(* $Id: asmlink.ml,v 1.70 2007/02/15 18:35:20 frisch Exp $ *)
 
 (* Link a set of .cmx/.o files and produce an executable *)
 
@@ -28,54 +28,61 @@ type error =
   | Assembler_error of string
   | Linking_error
   | Multiple_definition of string * string * string
+  | Missing_cmx of string * string
 
 exception Error of error
 
 (* Consistency check between interfaces and implementations *)
 
-let crc_interfaces =
-      (Hashtbl.create 17 : (string, string * Digest.t) Hashtbl.t)
-let crc_implementations =
-      (Hashtbl.create 17 : (string, string * Digest.t) Hashtbl.t)
+let crc_interfaces = Consistbl.create ()
+let crc_implementations = Consistbl.create ()
+let extra_implementations = ref ([] : string list)
+let implementations_defined = ref ([] : (string * string) list)
+let cmx_required = ref ([] : string list)
 
 let check_consistency file_name unit crc =
-  List.iter
-    (fun (name, crc) ->
-      if name = unit.ui_name then begin
-        Hashtbl.add crc_interfaces name (file_name, crc)
-      end else begin
-        try
-          let (auth_name, auth_crc) = Hashtbl.find crc_interfaces name in
-          if crc <> auth_crc then
-            raise(Error(Inconsistent_interface(name, file_name, auth_name)))
-        with Not_found ->
-          (* Can only happen for unit for which only a .cmi file was used,
-             but no .cmo is provided *)
-          Hashtbl.add crc_interfaces name (file_name, crc)
-      end)
-    unit.ui_imports_cmi;
-  List.iter
-    (fun (name, crc) ->
-      if crc <> cmx_not_found_crc then begin
-      try
-        let (auth_name, auth_crc) = Hashtbl.find crc_implementations name in
-        if crc <> auth_crc then
-          raise(Error(Inconsistent_implementation(name, file_name, auth_name)))
-      with Not_found ->
-        Hashtbl.add crc_implementations name (file_name, crc)
-      end)
-    unit.ui_imports_cmx;
-  try
-    let (name, crc) = Hashtbl.find crc_implementations unit.ui_name in
-    raise (Error(Multiple_definition(unit.ui_name, file_name, name)))
-  with Not_found ->
-    Hashtbl.add crc_implementations unit.ui_name (file_name, crc)
+  begin try
+    List.iter
+      (fun (name, crc) ->
+        if name = unit.ui_name
+        then Consistbl.set crc_interfaces name crc file_name
+        else Consistbl.check crc_interfaces name crc file_name)
+      unit.ui_imports_cmi
+  with Consistbl.Inconsistency(name, user, auth) ->
+    raise(Error(Inconsistent_interface(name, user, auth)))
+  end;
+  begin try
+    List.iter
+      (fun (name, crc) ->
+        if crc <> cmx_not_found_crc then
+          Consistbl.check crc_implementations name crc file_name
+        else if List.mem name !cmx_required then
+          raise(Error(Missing_cmx(file_name, name)))
+        else
+          extra_implementations := name :: !extra_implementations)
+      unit.ui_imports_cmx
+  with Consistbl.Inconsistency(name, user, auth) ->
+    raise(Error(Inconsistent_implementation(name, user, auth)))
+  end;
+  begin try
+    let source = List.assoc unit.ui_name !implementations_defined in
+    raise (Error(Multiple_definition(unit.ui_name, file_name, source)))
+  with Not_found -> ()
+  end;
+  Consistbl.set crc_implementations unit.ui_name crc file_name;
+  implementations_defined := 
+    (unit.ui_name, file_name) :: !implementations_defined;
+  if unit.ui_symbol <> unit.ui_name then
+    cmx_required := unit.ui_name :: !cmx_required
 
-let extract_crcs table =
-  Hashtbl.fold (fun name (file_name, crc) accu -> (name, crc) :: accu)
-               table []
-let extract_crc_interfaces () = extract_crcs crc_interfaces
-let extract_crc_implementations () = extract_crcs crc_implementations
+let extract_crc_interfaces () =
+  Consistbl.extract crc_interfaces
+let extract_crc_implementations () = 
+  List.fold_left
+    (fun ncl n ->
+      if List.mem_assoc n ncl then ncl else (n, cmx_not_found_crc) :: ncl)
+    (Consistbl.extract crc_implementations)
+    !extra_implementations
 
 (* Add C objects and options and "custom" info from a library descriptor.
    See bytecomp/bytelink.ml for comments on the order of C objects. *)
@@ -166,21 +173,24 @@ let make_startup_file ppf filename units_list =
   let compile_phrase p = Asmgen.compile_phrase ppf p in
   let oc = open_out filename in
   Emitaux.output_channel := oc;
-  Location.input_name := "startup"; (* set the name of the "current" input *)
-  Compilenv.reset "startup"; (* set the name of the "current" compunit *)
+  Location.input_name := "caml_startup"; (* set name of "current" input *)
+  Compilenv.reset "_startup"; (* set the name of the "current" compunit *)
   Emit.begin_assembly();
   let name_list =
     List.flatten (List.map (fun (info,_,_) -> info.ui_defines) units_list) in
   compile_phrase (Cmmgen.entry_point name_list);
   let apply_functions = ref (IntSet.add 2 (IntSet.add 3 IntSet.empty)) in
   (* The callback functions always reference caml_apply[23] *)
-  let curry_functions =
-    ref IntSet.empty in
+  let send_functions = ref IntSet.empty in
+  let curry_functions = ref IntSet.empty in
   List.iter
     (fun (info,_,_) ->
       List.iter
         (fun n -> apply_functions := IntSet.add n !apply_functions)
         info.ui_apply_fun;
+      List.iter
+        (fun n -> send_functions := IntSet.add n !send_functions)
+        info.ui_send_fun;
       List.iter
         (fun n -> curry_functions := IntSet.add n !curry_functions)
         info.ui_curry_fun)
@@ -188,6 +198,9 @@ let make_startup_file ppf filename units_list =
   IntSet.iter
     (fun n -> compile_phrase (Cmmgen.apply_function n))
     !apply_functions;
+  IntSet.iter
+    (fun n -> compile_phrase (Cmmgen.send_function n))
+    !send_functions;
   IntSet.iter
     (fun n -> List.iter (compile_phrase) (Cmmgen.curry_function n))
     !curry_functions;
@@ -202,10 +215,10 @@ let make_startup_file ppf filename units_list =
           try (unit.ui_name, List.assoc unit.ui_name unit.ui_imports_cmi)
           with Not_found -> assert false)
         units_list));
-  compile_phrase(Cmmgen.data_segment_table ("startup" :: name_list));
-  compile_phrase(Cmmgen.code_segment_table ("startup" :: name_list));
+  compile_phrase(Cmmgen.data_segment_table ("_startup" :: name_list));
+  compile_phrase(Cmmgen.code_segment_table ("_startup" :: name_list));
   compile_phrase
-    (Cmmgen.frame_table("startup" :: "system" :: name_list));
+    (Cmmgen.frame_table("_startup" :: "_system" :: name_list));
   Emit.end_assembly();
   close_out oc
 
@@ -222,13 +235,13 @@ let call_linker file_list startup_file output_name =
       raise(Error(File_not_found libname)) in
   let c_lib =
     if !Clflags.nopervasives then "" else Config.native_c_libraries in
-  let cmd =
-    match Config.ccomp_type with
-      "cc" ->
+  match Config.ccomp_type with
+  | "cc" ->
+      let cmd =
         if not !Clflags.output_c_object then
           Printf.sprintf "%s %s -o %s %s %s %s %s %s %s %s %s"
             !Clflags.c_linker
-            (if !Clflags.gprofile then "-pg" else "")
+            (if !Clflags.gprofile then Config.cc_profile else "")
             (Filename.quote output_name)
             (Clflags.std_include_flag "-I")
             (String.concat " " (List.rev !Clflags.ccopts))
@@ -246,8 +259,10 @@ let call_linker file_list startup_file output_name =
             (Filename.quote output_name)
             (Filename.quote startup_file)
             (Ccomp.quote_files (List.rev file_list))
-    | "msvc" ->
-        if not !Clflags.output_c_object then
+      in if Ccomp.command cmd <> 0 then raise(Error Linking_error)
+  | "msvc" ->
+      if not !Clflags.output_c_object then begin
+        let cmd =
           Printf.sprintf "%s /Fe%s %s %s %s %s %s %s %s"
             !Clflags.c_linker
             (Filename.quote output_name)
@@ -258,15 +273,19 @@ let call_linker file_list startup_file output_name =
               (List.rev_map Ccomp.expand_libname !Clflags.ccobjs))
             (Filename.quote runtime_lib)
             c_lib
-            (String.concat " " (List.rev !Clflags.ccopts))
-        else
+            (Ccomp.make_link_options !Clflags.ccopts) in
+        if Ccomp.command cmd <> 0 then raise(Error Linking_error);
+        if Ccomp.merge_manifest output_name <> 0 then raise(Error Linking_error)
+      end else begin
+        let cmd =
           Printf.sprintf "%s /out:%s %s %s"
             Config.native_partial_linker
             (Filename.quote output_name)
             (Filename.quote startup_file)
             (Ccomp.quote_files (List.rev file_list))
-    | _ -> assert false
-  in if Ccomp.command cmd <> 0 then raise(Error Linking_error)
+        in if Ccomp.command cmd <> 0 then raise(Error Linking_error)
+      end
+  | _ -> assert false
 
 let object_file_name name =
   let file_name =
@@ -357,3 +376,11 @@ let report_error ppf = function
       fprintf ppf
         "@[<hov>Files %s@ and %s@ both define a module named %s@]"
         file1 file2 modname
+  | Missing_cmx(filename, name) ->
+      fprintf ppf
+        "@[<hov>File %s@ was compiled without access@ \
+         to the .cmx file@ for module %s,@ \
+         which was produced by `ocamlopt -for-pack'.@ \
+         Please recompile %s@ with the correct `-I' option@ \
+         so that %s.cmx@ is found.@]"
+        filename name filename name
