@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: emitaux.mli,v 1.9 2000/02/21 19:38:06 xleroy Exp $ *)
+(* $Id: emitaux.mli,v 1.12 2007/01/29 12:10:50 xleroy Exp $ *)
 
 (* Common functions for emitting assembly code *)
 
@@ -18,9 +18,30 @@ val output_channel: out_channel ref
 val emit_string: string -> unit
 val emit_int: int -> unit
 val emit_nativeint: nativeint -> unit
+val emit_int32: int32 -> unit
 val emit_symbol: char -> string -> unit
 val emit_printf: ('a, out_channel, unit) format -> 'a
 val emit_char: char -> unit
 val emit_string_literal: string -> unit
 val emit_string_directive: string -> string -> unit
 val emit_bytes_directive: string -> string -> unit
+
+type frame_descr =
+  { fd_lbl: int;                        (* Return address *)
+    fd_frame_size: int;                 (* Size of stack frame *)
+    fd_live_offset: int list;           (* Offsets/regs of live addresses *)
+    fd_debuginfo: Debuginfo.t }         (* Location, if any *)
+
+val frame_descriptors : frame_descr list ref
+
+type emit_frame_actions =
+  { efa_label: int -> unit;
+    efa_16: int -> unit;
+    efa_32: int32 -> unit;
+    efa_word: int -> unit;
+    efa_align: int -> unit;
+    efa_label_rel: int -> int32 -> unit;
+    efa_def_label: int -> unit;
+    efa_string: string -> unit }
+
+val emit_frames: emit_frame_actions -> unit
