@@ -10,13 +10,19 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: arch.ml,v 1.6 2002/07/22 16:37:48 doligez Exp $ *)
+(* $Id: arch.ml,v 1.7 2002/11/29 15:03:36 xleroy Exp $ *)
 
 (* Specific operations for the HP PA-RISC processor *)
 
 open Misc
 open Format
 
+(* Machine-specific command-line options *)
+
+let command_line_options = []
+
+(* Specific operations *)
+
 type specific_operation =
     Ishift1add
   | Ishift2add
