@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: reg.ml,v 1.9 1999/11/17 18:56:35 xleroy Exp $ *)
+(* $Id: reg.ml,v 1.10 2000/06/29 11:44:06 xleroy Exp $ *)
 
 open Cmm
 
@@ -71,7 +71,17 @@ let at_location ty loc =
   incr currstamp;
   r
 
-let reset() = currstamp := 100; reg_list := []
+let first_virtual_reg_stamp = ref (-1)
+
+let reset() =
+  (* When reset() is called for the first time, the current stamp reflects
+     all hard pseudo-registers that have been allocated by Proc, so
+     remember it and use it as the base stamp for allocating 
+     soft pseudo-registers *)
+  if !first_virtual_reg_stamp = -1 then first_virtual_reg_stamp := !currstamp;
+  currstamp := !first_virtual_reg_stamp;
+  reg_list := []
+
 let all_registers() = !reg_list
 let num_registers() = !currstamp
 
