@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: spill.ml,v 1.15 1999/11/30 16:05:53 garrigue Exp $ *)
+(* $Id: spill.ml,v 1.19 2007/01/29 12:10:50 xleroy Exp $ *)
 
 (* Insertion of moves to suggest possible spilling / reloading points 
    before register allocation. *)
@@ -123,7 +123,14 @@ let add_reloads regset i =
     (fun r i -> instr_cons (Iop Ireload) [|spill_reg r|] [|r|] i)
     regset i
 
-let reload_at_exit = ref Reg.Set.empty
+let reload_at_exit = ref []
+
+let find_reload_at_exit k =
+  try
+    List.assoc k !reload_at_exit
+  with
+  | Not_found -> Misc.fatal_error "Spill.find_reload_at_exit"
+        
 let reload_at_break = ref Reg.Set.empty
 
 let rec reload i before =
@@ -140,7 +147,7 @@ let rec reload i before =
       (* All regs live across must be spilled *)
       let (new_next, finally) = reload i.next i.live in
       (add_reloads (Reg.inter_set_array before i.arg)
-                   (instr_cons i.desc i.arg i.res new_next),
+                   (instr_cons_debug i.desc i.arg i.res i.dbg new_next),
        finally)
   | Iop op ->
       let new_before =
@@ -153,7 +160,7 @@ let rec reload i before =
         Reg.diff_set_array (Reg.diff_set_array new_before i.arg) i.res in
       let (new_next, finally) = reload i.next after in
       (add_reloads (Reg.inter_set_array new_before i.arg)
-                   (instr_cons i.desc i.arg i.res new_next),
+                   (instr_cons_debug i.desc i.arg i.res i.dbg new_next),
        finally)
   | Iifthenelse(test, ifso, ifnot) ->
       let at_fork = Reg.diff_set_array before i.arg in
@@ -211,19 +218,20 @@ let rec reload i before =
       let (new_next, finally) = reload i.next Reg.Set.empty in
       (instr_cons (Iloop(!final_body)) i.arg i.res new_next,
        finally)
-  | Icatch(body, handler) ->
-      let saved_reload_at_exit = !reload_at_exit in
-      reload_at_exit := Reg.Set.empty;
+  | Icatch(nfail, body, handler) ->
+      let new_set = ref Reg.Set.empty in
+      reload_at_exit := (nfail, new_set) :: !reload_at_exit ;
       let (new_body, after_body) = reload body before in
-      let at_exit = !reload_at_exit in
-      reload_at_exit := saved_reload_at_exit;
+      let at_exit = !new_set in
+      reload_at_exit := List.tl !reload_at_exit ;
       let (new_handler, after_handler) = reload handler at_exit in
       let (new_next, finally) =
         reload i.next (Reg.Set.union after_body after_handler) in
-      (instr_cons (Icatch(new_body, new_handler)) i.arg i.res new_next,
+      (instr_cons (Icatch(nfail, new_body, new_handler)) i.arg i.res new_next,
        finally)
-  | Iexit ->
-      reload_at_exit := Reg.Set.union !reload_at_exit before;
+  | Iexit nfail ->
+      let set = find_reload_at_exit nfail in
+      set := Reg.Set.union !set before;
       (i, Reg.Set.empty)
   | Itrywith(body, handler) ->
       let (new_body, after_body) = reload body before in
@@ -239,15 +247,29 @@ let rec reload i before =
    That is, any register that may be reloaded in the future must be spilled
    just after its definition. *)
 
-(* As an optimization, if a register needs to be spilled in one branch of
+(*
+   As an optimization, if a register needs to be spilled in one branch of
    a conditional but not in the other, then we spill it late on entrance
    in the branch that needs it spilled.
-   This strategy is turned off in loops, as it may prevent a spill from
-   being lifted up all the way out of the loop. *)
+   NB: This strategy is turned off in loops, as it may prevent a spill from
+   being lifted up all the way out of the loop.
+   NB again: This strategy is also off in switch arms
+   as it generates many useless spills inside switch arms
+   NB ter: is it the same thing for catch bodies ?
+*)
+
+
+let spill_at_exit = ref []
+let find_spill_at_exit k =
+  try
+    List.assoc k !spill_at_exit
+  with
+  | Not_found -> Misc.fatal_error "Spill.find_spill_at_exit"
 
-let spill_at_exit = ref Reg.Set.empty
 let spill_at_raise = ref Reg.Set.empty
 let inside_loop = ref false
+and inside_arm = ref false
+and inside_catch = ref false
 
 let add_spills regset i =
   Reg.Set.fold
@@ -270,19 +292,21 @@ let rec spill i finally =
       let before1 = Reg.diff_set_array after i.res in
       let before =
         match i.desc with
-          Iop(Icall_ind) | Iop(Icall_imm _) | Iop(Iextcall(_, _))
+          Iop Icall_ind | Iop(Icall_imm _) | Iop(Iextcall _)
         | Iop(Iintop Icheckbound) | Iop(Iintop_imm(Icheckbound, _)) ->
             Reg.Set.union before1 !spill_at_raise
         | _ ->
             before1 in
-      (instr_cons i.desc i.arg i.res
+      (instr_cons_debug i.desc i.arg i.res i.dbg
                   (add_spills (Reg.inter_set_array after i.res) new_next),
        before)
   | Iifthenelse(test, ifso, ifnot) ->
       let (new_next, at_join) = spill i.next finally in
       let (new_ifso, before_ifso) = spill ifso at_join in
       let (new_ifnot, before_ifnot) = spill ifnot at_join in
-      if !inside_loop then
+      if
+        !inside_loop || !inside_arm
+      then
         (instr_cons (Iifthenelse(test, new_ifso, new_ifnot))
                      i.arg i.res new_next,
          Reg.Set.union before_ifso before_ifnot)
@@ -302,6 +326,8 @@ let rec spill i finally =
       end
   | Iswitch(index, cases) ->
       let (new_next, at_join) = spill i.next finally in
+      let saved_inside_arm = !inside_arm in
+      inside_arm := true ;
       let before = ref Reg.Set.empty in
       let new_cases =
         Array.map
@@ -310,6 +336,7 @@ let rec spill i finally =
             before := Reg.Set.union !before before_c;
             new_c)
           cases in
+      inside_arm := saved_inside_arm ;
       (instr_cons (Iswitch(index, new_cases)) i.arg i.res new_next,
        !before)
   | Iloop(body) ->
@@ -332,17 +359,19 @@ let rec spill i finally =
       inside_loop := saved_inside_loop;
       (instr_cons (Iloop(!final_body)) i.arg i.res new_next,
        !at_head)
-  | Icatch(body, handler) ->
+  | Icatch(nfail, body, handler) ->
       let (new_next, at_join) = spill i.next finally in
       let (new_handler, at_exit) = spill handler at_join in
-      let saved_spill_at_exit = !spill_at_exit in
-      spill_at_exit := at_exit;
+      let saved_inside_catch = !inside_catch in
+      inside_catch := true ;
+      spill_at_exit := (nfail, at_exit) :: !spill_at_exit ;
       let (new_body, before) = spill body at_join in
-      spill_at_exit := saved_spill_at_exit;
-      (instr_cons (Icatch(new_body, new_handler)) i.arg i.res new_next,
+      spill_at_exit := List.tl !spill_at_exit;
+      inside_catch := saved_inside_catch ;
+      (instr_cons (Icatch(nfail, new_body, new_handler)) i.arg i.res new_next,
        before)
-  | Iexit ->
-      (i, !spill_at_exit)
+  | Iexit nfail ->
+      (i, find_spill_at_exit nfail)
   | Itrywith(body, handler) ->
       let (new_next, at_join) = spill i.next finally in
       let (new_handler, before_handler) = spill handler at_join in
