@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: signals_osdep.h,v 1.3.2.1 2005/12/30 08:40:56 xleroy Exp $ */
+/* $Id: signals_osdep.h,v 1.8 2007/01/29 12:10:52 xleroy Exp $ */
 
 /* Processor- and OS-dependent signal interface */
 
@@ -70,11 +70,28 @@
     static void name(int sig, siginfo_t * info, void * context)
 
   #define SET_SIGACT(sigact,name) \
-     sigact.sa_sigaction = (name);
+     sigact.sa_sigaction = (name); \
      sigact.sa_flags = SA_SIGINFO
 
   #define CONTEXT_FAULTING_ADDRESS ((char *) info->si_addr)
 
+/****************** I386, MacOS X */
+
+#elif defined(TARGET_i386) && defined(SYS_macosx)
+
+  #define DECLARE_SIGNAL_HANDLER(name) \
+    static void name(int sig, siginfo_t * info, void * context)
+
+  #define SET_SIGACT(sigact,name) \
+     sigact.sa_sigaction = (name); \
+     sigact.sa_flags = SA_SIGINFO
+
+  #include <sys/ucontext.h>
+
+  #define CONTEXT_STATE (((struct ucontext *)context)->uc_mcontext->ss)
+  #define CONTEXT_PC (CONTEXT_STATE.eip)
+  #define CONTEXT_FAULTING_ADDRESS ((char *) info->si_addr)
+
 /****************** MIPS, all OS */
 
 #elif defined(TARGET_mips)
@@ -96,20 +113,47 @@
 
 #elif defined(TARGET_power) && defined(SYS_rhapsody)
 
+#ifdef __ppc64__
+
+  #define DECLARE_SIGNAL_HANDLER(name) \
+     static void name(int sig, siginfo_t * info, void * context)
+
+  #define SET_SIGACT(sigact,name) \
+     sigact.sa_sigaction = (name); \
+     sigact.sa_flags = SA_SIGINFO | SA_64REGSET
+
+  typedef unsigned long long context_reg;
+
+  #include <sys/ucontext.h>
+
+  #define CONTEXT_STATE (((struct ucontext64 *)context)->uc_mcontext64->ss)
+
+  #define CONTEXT_PC (CONTEXT_STATE.srr0)
+  #define CONTEXT_EXCEPTION_POINTER (CONTEXT_STATE.r29)
+  #define CONTEXT_YOUNG_LIMIT (CONTEXT_STATE.r30)
+  #define CONTEXT_YOUNG_PTR (CONTEXT_STATE.r31)
+  #define CONTEXT_FAULTING_ADDRESS ((char *) info->si_addr)
+  #define CONTEXT_SP (CONTEXT_STATE.r1)
+
+#else
+
   #include <sys/utsname.h>
 
   #define DECLARE_SIGNAL_HANDLER(name) \
-     static void name(int sig, int code, void * context)
+     static void name(int sig, siginfo_t * info, void * context)
 
   #define SET_SIGACT(sigact,name) \
      sigact.sa_handler = (void (*)(int)) (name); \
      sigact.sa_flags = SA_SIGINFO
 
   typedef unsigned long context_reg;
+
   #define CONTEXT_PC (*context_gpr_p(context, -2))
   #define CONTEXT_EXCEPTION_POINTER (*context_gpr_p(context, 29))
   #define CONTEXT_YOUNG_LIMIT (*context_gpr_p(context, 30))
   #define CONTEXT_YOUNG_PTR (*context_gpr_p(context, 31))
+  #define CONTEXT_FAULTING_ADDRESS ((char *) info->si_addr)
+  #define CONTEXT_SP (*context_gpr_p(context, 1))
 
   static int ctx_version = 0;
   static void init_ctx (void)
@@ -168,6 +212,8 @@
     }
   #endif
 
+#endif
+
 /****************** PowerPC, ELF (Linux) */
 
 #elif defined(TARGET_power) && defined(SYS_elf)
