@@ -10,13 +10,13 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: bytelibrarian.ml,v 1.15 2001/10/30 09:32:29 xleroy Exp $ *)
+(* $Id: bytelibrarian.ml,v 1.19 2006/05/11 15:50:53 xleroy Exp $ *)
 
 (* Build libraries of .cmo files *)
 
 open Misc
 open Config
-open Emitcode
+open Cmo_format
 
 type error =
     File_not_found of string
@@ -69,6 +69,7 @@ let copy_object_file oc name =
       let compunit_pos = input_binary_int ic in
       seek_in ic compunit_pos;
       let compunit = (input_value ic : compilation_unit) in
+      Bytelink.check_consistency file_name compunit;
       copy_compunit ic oc compunit;
       close_in ic;
       [compunit]
@@ -77,6 +78,7 @@ let copy_object_file oc name =
       let toc_pos = input_binary_int ic in
       seek_in ic toc_pos;
       let toc = (input_value ic : library) in
+      List.iter (Bytelink.check_consistency file_name) toc.lib_units;
       add_ccobjs toc;
       List.iter (copy_compunit ic oc) toc.lib_units;
       close_in ic;
