@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: bytelibrarian.mli,v 1.5 1999/11/17 18:56:59 xleroy Exp $ *)
+(* $Id: bytelibrarian.mli,v 1.6 2000/03/06 22:11:06 weis Exp $ *)
 
 (* Build libraries of .cmo files *)
 
@@ -29,4 +29,6 @@ type error =
 
 exception Error of error
 
-val report_error: error -> unit
+open Format
+
+val report_error: formatter -> error -> unit
