@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: translcore.ml,v 1.100 2005/08/25 15:35:16 doligez Exp $ *)
+(* $Id: translcore.ml,v 1.102 2007/02/09 13:31:15 doligez Exp $ *)
 
 (* Translation from typed abstract syntax to lambda terms,
    for the core language *)
@@ -250,12 +250,12 @@ let primitives_table = create_hashtable 57 [
   "%int64_to_int32", Pcvtbint(Pint64, Pint32);
   "%int64_of_nativeint", Pcvtbint(Pnativeint, Pint64);
   "%int64_to_nativeint", Pcvtbint(Pint64, Pnativeint);
-  "%bigarray_ref_1", Pbigarrayref(1, Pbigarray_unknown, Pbigarray_c_layout);
-  "%bigarray_ref_2", Pbigarrayref(2, Pbigarray_unknown, Pbigarray_c_layout);
-  "%bigarray_ref_3", Pbigarrayref(3, Pbigarray_unknown, Pbigarray_c_layout);
-  "%bigarray_set_1", Pbigarrayset(1, Pbigarray_unknown, Pbigarray_c_layout);
-  "%bigarray_set_2", Pbigarrayset(2, Pbigarray_unknown, Pbigarray_c_layout);
-  "%bigarray_set_3", Pbigarrayset(3, Pbigarray_unknown, Pbigarray_c_layout)
+  "%caml_ba_ref_1", Pbigarrayref(1, Pbigarray_unknown, Pbigarray_c_layout);
+  "%caml_ba_ref_2", Pbigarrayref(2, Pbigarray_unknown, Pbigarray_c_layout);
+  "%caml_ba_ref_3", Pbigarrayref(3, Pbigarray_unknown, Pbigarray_c_layout);
+  "%caml_ba_set_1", Pbigarrayset(1, Pbigarray_unknown, Pbigarray_c_layout);
+  "%caml_ba_set_2", Pbigarrayset(2, Pbigarray_unknown, Pbigarray_c_layout);
+  "%caml_ba_set_3", Pbigarrayset(3, Pbigarray_unknown, Pbigarray_c_layout)
 ]
 
 let prim_makearray =
@@ -368,6 +368,7 @@ let check_recursive_lambda idlist lam =
         List.for_all (check idlist) args
     | Lprim(Pmakearray(Paddrarray|Pintarray), args) ->
         List.for_all (check idlist) args
+    | Lprim (Pmakearray (Pgenarray), args) -> false
     | Lsequence (lam1, lam2) -> check idlist lam1 && check idlist lam2
     | Levent (lam, _) -> check idlist lam
     | lam ->
@@ -385,9 +386,10 @@ let check_recursive_lambda idlist lam =
                     bindings idlist
 
   (* reverse-engineering the code generated by transl_record case 2 *)
+  (* If you change this, you probably need to change Bytegen.size_of_lambda. *)
   and check_recursive_recordwith idlist = function
-    | Llet (Strict, id1, Lprim (Pccall prim, [e1]), body) ->
-       prim = prim_obj_dup && check_top idlist e1
+    | Llet (Strict, id1, Lprim (Pduprecord _, [e1]), body) ->
+       check_top idlist e1
        && check_recordwith_updates idlist id1 body
     | _ -> false
 
@@ -491,7 +493,7 @@ let primitive_is_ccall = function
   (* Determine if a primitive is a Pccall or will be turned later into
      a C function call that may raise an exception *)
   | Pccall _ | Pstringrefs | Pstringsets | Parrayrefs _ | Parraysets _ |
-    Pbigarrayref _ | Pbigarrayset _ -> true
+    Pbigarrayref _ | Pbigarrayset _ | Pduprecord _ -> true
   | _ -> false
 
 (* Assertions *)
@@ -536,15 +538,15 @@ and transl_exp0 e =
       let public_send = p.prim_name = "%send" in
       if public_send || p.prim_name = "%sendself" then
         let kind = if public_send then Public else Self in
-	let obj = Ident.create "obj" and meth = Ident.create "meth" in
-	Lfunction(Curried, [obj; meth], Lsend(kind, Lvar meth, Lvar obj, []))
+        let obj = Ident.create "obj" and meth = Ident.create "meth" in
+        Lfunction(Curried, [obj; meth], Lsend(kind, Lvar meth, Lvar obj, []))
       else if p.prim_name = "%sendcache" then
-	let obj = Ident.create "obj" and meth = Ident.create "meth" in
+        let obj = Ident.create "obj" and meth = Ident.create "meth" in
         let cache = Ident.create "cache" and pos = Ident.create "pos" in
-	Lfunction(Curried, [obj; meth; cache; pos],
+        Lfunction(Curried, [obj; meth; cache; pos],
                   Lsend(Cached, Lvar meth, Lvar obj, [Lvar cache; Lvar pos]))
       else
-	transl_primitive p
+        transl_primitive p
   | Texp_ident(path, {val_kind = Val_anc _}) ->
       raise(Error(e.exp_loc, Free_super_var))
   | Texp_ident(path, {val_kind = Val_reg | Val_self _}) ->
@@ -576,8 +578,8 @@ and transl_exp0 e =
         || not !Clflags.native_code && p.prim_name = "%sendcache"in
       if public_send || p.prim_name = "%sendself" then
         let kind = if public_send then Public else Self in
-	let obj = List.hd argl in
-	wrap (Lsend (kind, List.nth argl 1, obj, []))
+        let obj = List.hd argl in
+        wrap (Lsend (kind, List.nth argl 1, obj, []))
       else if p.prim_name = "%sendcache" then
         match argl with [obj; meth; cache; pos] ->
           wrap (Lsend(Cached, meth, obj, [cache; pos]))
@@ -662,7 +664,7 @@ and transl_exp0 e =
         let cl = List.map extract_constant ll in
         let master =
           match kind with
-          | Paddrarray | Pintarray ->        
+          | Paddrarray | Pintarray ->
               Lconst(Const_block(0, cl))
           | Pfloatarray ->
               Lconst(Const_float_array(List.map extract_float cl))
@@ -694,11 +696,11 @@ and transl_exp0 e =
   | Texp_send(expr, met) ->
       let obj = transl_exp expr in
       let lam =
-        match met with 
+        match met with
           Tmeth_val id -> Lsend (Self, Lvar id, obj, [])
         | Tmeth_name nm ->
             let (tag, cache) = Translobj.meth obj nm in
-	    let kind = if cache = [] then Public else Cached in
+            let kind = if cache = [] then Public else Cached in
             Lsend (kind, tag, obj, cache)
       in
       event_after e lam
@@ -856,8 +858,9 @@ and transl_setinstvar self var expr =
                     [self; transl_path var; transl_exp expr])
 
 and transl_record all_labels repres lbl_expr_list opt_init_expr =
+  let size = Array.length all_labels in
   (* Determine if there are "enough" new fields *)
-  if 3 + 2 * List.length lbl_expr_list >= Array.length all_labels
+  if 3 + 2 * List.length lbl_expr_list >= size
   then begin
     (* Allocate new record with given fields (and remaining fields
        taken from init_expr if any *)
@@ -914,7 +917,7 @@ and transl_record all_labels repres lbl_expr_list opt_init_expr =
       None -> assert false
     | Some init_expr ->
         Llet(Strict, copy_id,
-             Lprim(Pccall prim_obj_dup, [transl_exp init_expr]),
+             Lprim(Pduprecord (repres, size), [transl_exp init_expr]),
              List.fold_right update_field lbl_expr_list (Lvar copy_id))
     end
   end
