@@ -11,27 +11,27 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: debugger.h,v 1.7 2001/12/07 13:39:24 xleroy Exp $ */
+/* $Id: debugger.h,v 1.10 2005/09/22 14:21:50 xleroy Exp $ */
 
 /* Interface with the debugger */
 
-#ifndef _debugger_
-#define _debugger_
+#ifndef CAML_DEBUGGER_H
+#define CAML_DEBUGGER_H
 
 #include "misc.h"
 #include "mlvalues.h"
 
-extern int debugger_in_use;
+extern int caml_debugger_in_use;
 extern int running;
-extern unsigned long event_count;
+extern uintnat caml_event_count;
 
 enum event_kind {
   EVENT_COUNT, BREAKPOINT, PROGRAM_START, PROGRAM_EXIT,
   TRAP_BARRIER, UNCAUGHT_EXC
 };
 
-void debugger_init (void);
-void debugger (enum event_kind event);
+void caml_debugger_init (void);
+void caml_debugger (enum event_kind event);
 
 /* Communication protocol */
 
@@ -83,7 +83,7 @@ enum debugger_request {
   /* As REQ_GET_OBJ, but sends only one field. */
   REQ_MARSHAL_OBJ = 'M',        /* mlvalue v */
   /* Send a copy of the data structure rooted at v, using the same
-     format as output_value. */
+     format as [caml_output_value]. */
   REQ_GET_CLOSURE_CODE = 'C'    /* mlvalue v */
   /* Send the code address of the given closure.
      Reply is one uint32. */
@@ -107,6 +107,4 @@ enum debugger_reply {
   /* Program exited due to a stray exception. */
 };
 
-#endif
-
-
+#endif /* CAML_DEBUGGER_H */
