@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: main.c,v 1.31 2001/12/07 13:39:31 xleroy Exp $ */
+/* $Id: main.c,v 1.36 2004/01/08 22:28:48 doligez Exp $ */
 
 /* Main entry point (can be overridden by a user-provided main()
    function that calls caml_main() later). */
@@ -20,40 +20,19 @@
 #include "mlvalues.h"
 #include "sys.h"
 
-extern void caml_main (char **);
-
-#ifdef __linux__
-#include <unistd.h>
-#endif
+CAMLextern void caml_main (char **);
 
 #ifdef _WIN32
-extern void expand_command_line (int *, char ***);
-#endif
-
-#if macintosh
-#include "rotatecursor.h"
-#include "signals.h"
+CAMLextern void caml_expand_command_line (int *, char ***);
 #endif
 
 int main(int argc, char **argv)
 {
 #ifdef _WIN32
   /* Expand wildcards and diversions in command line */
-  expand_command_line(&argc, &argv);
+  caml_expand_command_line(&argc, &argv);
 #endif
-#ifdef __linux__
-  /* Recover argv[0] from /proc/self/exe, much more reliable */
-  char exename[1024];
-  int retcode = readlink("/proc/self/exe", exename, sizeof(exename));
-  if (retcode != -1 && retcode < sizeof(exename)) {
-    exename[retcode] = 0;
-    argv[0] = exename;
-  }
-#endif
-#if macintosh
-  rotatecursor_options (&something_to_do, 0, NULL);
-#endif /* macintosh */
   caml_main(argv);
-  sys_exit(Val_int(0));
+  caml_sys_exit(Val_int(0));
   return 0; /* not reached */
 }
