@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: unix.c,v 1.25 2005/09/22 14:21:50 xleroy Exp $ */
+/* $Id: unix.c,v 1.28 2007/02/12 07:57:25 weis Exp $ */
 
 /* Unix-specific stuff */
 
@@ -199,7 +199,7 @@ entry_t *caml_lookup_bundle(const char *name)
   return current;
 }
 
-void * caml_dlopen(char * libname)
+void * caml_dlopen(char * libname, int for_execution)
 {
   NSObjectFileImage image;
   entry_t *bentry = caml_lookup_bundle(libname);
@@ -283,9 +283,10 @@ char * caml_dlerror(void)
 #define RTLD_NODELETE 0
 #endif
 
-void * caml_dlopen(char * libname)
+void * caml_dlopen(char * libname, int for_execution)
 {
-  return dlopen(libname, RTLD_NOW|RTLD_GLOBAL|RTLD_NODELETE);
+  return dlopen(libname, RTLD_NOW | RTLD_GLOBAL | RTLD_NODELETE);
+  /* Could use RTLD_LAZY if for_execution == 0, but needs testing */
 }
 
 void caml_dlclose(void * handle)
@@ -311,7 +312,7 @@ char * caml_dlerror(void)
 #endif
 #else
 
-void * caml_dlopen(char * libname)
+void * caml_dlopen(char * libname, int for_execution)
 {
   return NULL;
 }
