@@ -1,37 +1,53 @@
+;(***********************************************************************)
+;(*                                                                     *)
+;(*                           Objective Caml                            *)
+;(*                                                                     *)
+;(*                Jacques Garrigue and Ian T Zimmerman                 *)
+;(*                                                                     *)
+;(*  Copyright 1997 Institut National de Recherche en Informatique et   *)
+;(*  en Automatique.  All rights reserved.  This file is distributed    *)
+;(*  under the terms of the GNU General Public License.                 *)
+;(*                                                                     *)
+;(***********************************************************************)
+
+;(* $Id: caml-font.el,v 1.19 2004/08/20 17:04:35 doligez Exp $ *)
+
 ;; useful colors
 
 (cond
  ((x-display-color-p)
-;
-;       (not (memq 'font-lock-type-face (face-list))))
-  ; make the necessary faces
-  (make-face 'Firebrick)
-  (set-face-foreground 'Firebrick "Firebrick")
-  (make-face 'RosyBrown)
-  (set-face-foreground 'RosyBrown "RosyBrown")
-  (make-face 'Purple)
-  (set-face-foreground 'Purple "Purple")
-  (make-face 'MidnightBlue)
-  (set-face-foreground 'MidnightBlue "MidnightBlue")
-  (make-face 'DarkGoldenRod)
-  (set-face-foreground 'DarkGoldenRod "DarkGoldenRod")
-  (make-face 'DarkOliveGreen)
-  (set-face-foreground 'DarkOliveGreen "DarkOliveGreen4")
-  (make-face 'CadetBlue)
-  (set-face-foreground 'CadetBlue "CadetBlue")
+  (require 'font-lock)
+  (cond
+   ((not (boundp 'font-lock-type-face))
+    ; make the necessary faces
+    (make-face 'Firebrick)
+    (set-face-foreground 'Firebrick "Firebrick")
+    (make-face 'RosyBrown)
+    (set-face-foreground 'RosyBrown "RosyBrown")
+    (make-face 'Purple)
+    (set-face-foreground 'Purple "Purple")
+    (make-face 'MidnightBlue)
+    (set-face-foreground 'MidnightBlue "MidnightBlue")
+    (make-face 'DarkGoldenRod)
+    (set-face-foreground 'DarkGoldenRod "DarkGoldenRod")
+    (make-face 'DarkOliveGreen)
+    (set-face-foreground 'DarkOliveGreen "DarkOliveGreen4")
+    (make-face 'CadetBlue)
+    (set-face-foreground 'CadetBlue "CadetBlue")
+    ; assign them as standard faces
+    (setq font-lock-comment-face 'Firebrick)
+    (setq font-lock-string-face 'RosyBrown)
+    (setq font-lock-keyword-face 'Purple)
+    (setq font-lock-function-name-face 'MidnightBlue)
+    (setq font-lock-variable-name-face 'DarkGoldenRod)
+    (setq font-lock-type-face 'DarkOliveGreen)
+    (setq font-lock-reference-face 'CadetBlue)))
+  ; extra faces for documention
   (make-face 'Stop)
   (set-face-foreground 'Stop "White")
   (set-face-background 'Stop "Red")
   (make-face 'Doc)
   (set-face-foreground 'Doc "Red")
-  ; assign them as standard faces
-  (setq font-lock-comment-face 'Firebrick)
-  (setq font-lock-string-face 'RosyBrown)
-  (setq font-lock-keyword-face 'Purple)
-  (setq font-lock-function-name-face 'MidnightBlue)
-  (setq font-lock-variable-name-face 'DarkGoldenRod)
-  (setq font-lock-type-face 'DarkOliveGreen)
-  (setq font-lock-reference-face 'CadetBlue)
   (setq font-lock-stop-face 'Stop)
   (setq font-lock-doccomment-face 'Doc)
 ))
@@ -66,7 +82,7 @@
           "\\|in\\(herit\\|itializer\\)?\\|let"
           "\\|m\\(ethod\\|utable\\|odule\\)"
           "\\|of\\|p\\(arser\\|rivate\\)\\|rec\\|type"
-          "\\|v\\(al\\(ue\\)?\\|irtual\\)\\)\\>")
+          "\\|v\\(al\\|irtual\\)\\)\\>")
          'font-lock-type-face)
 ;blocking
    '("\\<\\(begin\\|end\\|object\\|s\\(ig\\|truct\\)\\)\\>"
@@ -93,7 +109,7 @@
    caml-font-lock-keywords))
 
 ;; font-lock commands are similar for caml-mode and inferior-caml-mode
-(setq caml-mode-hook
+(add-hook 'caml-mode-hook
       '(lambda ()
          (cond
           ((fboundp 'global-font-lock-mode)
