@@ -10,15 +10,15 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: output.mli,v 1.6 2002/01/04 10:01:49 ddr Exp $ *)
+(* $Id: output.mli,v 1.8 2002/12/09 10:44:46 maranget Exp $ *)
 
 (* Output the DFA tables and its entry points *)
 
 val output_lexdef:
-      string -> in_channel -> out_channel ->
+      string -> in_channel -> out_channel -> Common.line_tracker ->
       Syntax.location ->
       Compact.lex_tables ->
-      (unit, Syntax.location) Lexgen.automata_entry list ->
+      (string list, Syntax.location) Lexgen.automata_entry list ->
       Syntax.location ->
       unit
 
