@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc_cross.ml,v 1.15.10.1 2005/11/07 15:59:04 doligez Exp $ *)
+(* $Id: odoc_cross.ml,v 1.17 2006/09/20 11:14:36 doligez Exp $ *)
 
 (** Cross referencing. *)
 
@@ -87,8 +87,8 @@ let rec build_alias_list = function
       (
        match m.m_kind with
          Module_alias ma ->
-	   Hashtbl.add module_aliases m.m_name (ma.ma_name, Alias_to_resolve);
-	   Hashtbl.add module_and_modtype_aliases m.m_name (ma.ma_name, Alias_to_resolve)
+           Hashtbl.add module_aliases m.m_name (ma.ma_name, Alias_to_resolve);
+           Hashtbl.add module_and_modtype_aliases m.m_name (ma.ma_name, Alias_to_resolve)
        | _ -> ()
       );
       build_alias_list q
@@ -96,8 +96,8 @@ let rec build_alias_list = function
       (
        match mt.mt_kind with
          Some (Module_type_alias mta) ->
-	   Hashtbl.add module_and_modtype_aliases
-	     mt.mt_name (mta.mta_name, Alias_to_resolve)
+           Hashtbl.add module_and_modtype_aliases
+             mt.mt_name (mta.mta_name, Alias_to_resolve)
        | _ -> ()
       );
       build_alias_list q
@@ -106,8 +106,8 @@ let rec build_alias_list = function
        match e.ex_alias with
          None -> ()
        | Some ea ->
-	   Hashtbl.add exception_aliases
-	     e.ex_name (ea.ea_name,Alias_to_resolve)
+           Hashtbl.add exception_aliases
+             e.ex_name (ea.ea_name,Alias_to_resolve)
       );
       build_alias_list q
   | _ :: q ->
@@ -126,26 +126,26 @@ let name_alias =
   let rec f t name =
     try
       match Hashtbl.find t name with
-	(s, Alias_resolved) -> s
-      |	(s, Alias_to_resolve) -> f t s
+        (s, Alias_resolved) -> s
+      | (s, Alias_to_resolve) -> f t s
     with
       Not_found ->
-	try
-	  Hashtbl.iter
-	    (fun n2 (n3, _) ->
-	      if Name.prefix n2 name then
-		let ln2 = String.length n2 in
-		let s = n3^(String.sub name ln2 ((String.length name) - ln2)) in
-		raise (Found s)
-	    )
-	    t ;
-	  Hashtbl.replace t name (name, Alias_resolved);
-	  name
-	with
-	  Found s ->
-	    let s2 = f t s in
-	    Hashtbl.replace t s2 (s2, Alias_resolved);
-	    s2
+        try
+          Hashtbl.iter
+            (fun n2 (n3, _) ->
+              if Name.prefix n2 name then
+                let ln2 = String.length n2 in
+                let s = n3^(String.sub name ln2 ((String.length name) - ln2)) in
+                raise (Found s)
+            )
+            t ;
+          Hashtbl.replace t name (name, Alias_resolved);
+          name
+        with
+          Found s ->
+            let s2 = f t s in
+            Hashtbl.replace t s2 (s2, Alias_resolved);
+            s2
   in
   fun name alias_tbl ->
     f alias_tbl name
@@ -169,7 +169,7 @@ let add_known_element name k =
     Not_found ->
       known_elements := Ele_map.add name [k] !known_elements
 
-let get_known_elements name =
+let rec get_known_elements name =
   try Ele_map.find name !known_elements
   with Not_found -> []
 
@@ -252,10 +252,10 @@ class scan =
       add_known_element e.ex_name (Odoc_search.Res_exception e)
     method scan_attribute a =
       add_known_element a.att_value.val_name
-	(Odoc_search.Res_attribute a)
+        (Odoc_search.Res_attribute a)
     method scan_method m =
       add_known_element m.met_value.val_name
-	(Odoc_search.Res_method m)
+        (Odoc_search.Res_method m)
     method scan_class_pre c =
       add_known_element c.cl_name (Odoc_search.Res_class c);
       true
@@ -576,118 +576,165 @@ and associate_in_class_type module_list (acc_b_modif, acc_incomplete_top_module_
 
 let ao = Odoc_misc.apply_opt
 
-let rec assoc_comments_text_elements module_list t_ele =
+let not_found_of_kind kind name =
+  (match kind with
+    RK_module -> Odoc_messages.cross_module_not_found
+  | RK_module_type -> Odoc_messages.cross_module_type_not_found
+  | RK_class -> Odoc_messages.cross_class_not_found
+  | RK_class_type -> Odoc_messages.cross_class_type_not_found
+  | RK_value -> Odoc_messages.cross_value_not_found
+  | RK_type -> Odoc_messages.cross_type_not_found
+  | RK_exception -> Odoc_messages.cross_exception_not_found
+  | RK_attribute -> Odoc_messages.cross_attribute_not_found
+  | RK_method -> Odoc_messages.cross_method_not_found
+  | RK_section _ -> Odoc_messages.cross_section_not_found
+  ) name
+
+let rec assoc_comments_text_elements parent_name module_list t_ele =
   match t_ele with
   | Raw _
   | Code _
   | CodePre _
   | Latex _
   | Verbatim _ -> t_ele
-  | Bold t -> Bold (assoc_comments_text module_list t)
-  | Italic t -> Italic (assoc_comments_text module_list t)
-  | Center t -> Center (assoc_comments_text module_list t)
-  | Left t -> Left (assoc_comments_text module_list t)
-  | Right t -> Right (assoc_comments_text module_list t)
-  | Emphasize t -> Emphasize (assoc_comments_text module_list t)
-  | List l -> List (List.map (assoc_comments_text module_list) l)
-  | Enum l -> Enum (List.map (assoc_comments_text module_list) l)
+  | Bold t -> Bold (assoc_comments_text parent_name module_list t)
+  | Italic t -> Italic (assoc_comments_text parent_name module_list t)
+  | Center t -> Center (assoc_comments_text parent_name module_list t)
+  | Left t -> Left (assoc_comments_text parent_name module_list t)
+  | Right t -> Right (assoc_comments_text parent_name module_list t)
+  | Emphasize t -> Emphasize (assoc_comments_text parent_name module_list t)
+  | List l -> List (List.map (assoc_comments_text parent_name module_list) l)
+  | Enum l -> Enum (List.map (assoc_comments_text parent_name module_list) l)
   | Newline -> Newline
-  | Block t -> Block (assoc_comments_text module_list t)
-  | Superscript t -> Superscript (assoc_comments_text module_list t)
-  | Subscript t -> Subscript (assoc_comments_text module_list t)
-  | Title (n, l_opt, t) -> Title (n, l_opt, (assoc_comments_text module_list t))
-  | Link (s, t) -> Link (s, (assoc_comments_text module_list t))
-  | Ref (name, None) ->
+  | Block t -> Block (assoc_comments_text parent_name module_list t)
+  | Superscript t -> Superscript (assoc_comments_text parent_name module_list t)
+  | Subscript t -> Subscript (assoc_comments_text parent_name module_list t)
+  | Title (n, l_opt, t) -> Title (n, l_opt, (assoc_comments_text parent_name module_list t))
+  | Link (s, t) -> Link (s, (assoc_comments_text parent_name module_list t))
+  | Ref (initial_name, None) ->
       (
-       match get_known_elements name with
-	 [] ->
-	   (
-	    try
-	      let re = Str.regexp ("^"^(Str.quote name)^"$") in
-	      let t = Odoc_search.find_section module_list re in
-	      let v2 = (name, Some (RK_section t)) in
-	      add_verified v2 ;
-	      Ref (name, Some (RK_section t))
-	    with
-	      Not_found ->
-		Odoc_messages.pwarning (Odoc_messages.cross_element_not_found name);
-		Ref (name, None)
-	   )
-       | ele :: _ ->
-	   (* we look for the first element with this name *)
-           let kind =
-             match ele with
-               Odoc_search.Res_module _ -> RK_module
-             | Odoc_search.Res_module_type _ -> RK_module_type
-             | Odoc_search.Res_class _ -> RK_class
-             | Odoc_search.Res_class_type _ -> RK_class_type
-             | Odoc_search.Res_value _ -> RK_value
-             | Odoc_search.Res_type _ -> RK_type
-             | Odoc_search.Res_exception _ -> RK_exception
-             | Odoc_search.Res_attribute _ -> RK_attribute
-             | Odoc_search.Res_method _ -> RK_method
-             | Odoc_search.Res_section (_ ,t)-> assert false
-           in
-           add_verified (name, Some kind) ;
-	   Ref (name, Some kind)
+       let rec iter_parent ?parent_name name =
+         let res =
+           match get_known_elements name with
+             [] ->
+               (
+                try
+                  let re = Str.regexp ("^"^(Str.quote name)^"$") in
+                  let t = Odoc_search.find_section module_list re in
+                  let v2 = (name, Some (RK_section t)) in
+                  add_verified v2 ;
+                  (name, Some (RK_section t))
+              with
+                  Not_found ->
+                    (name, None)
+               )
+           | ele :: _ ->
+           (* we look for the first element with this name *)
+               let (name, kind) =
+                 match ele with
+                   Odoc_search.Res_module m -> (m.m_name, RK_module)
+                 | Odoc_search.Res_module_type mt -> (mt.mt_name, RK_module_type)
+                 | Odoc_search.Res_class c -> (c.cl_name, RK_class)
+                 | Odoc_search.Res_class_type ct -> (ct.clt_name, RK_class_type)
+                 | Odoc_search.Res_value v -> (v.val_name, RK_value)
+                 | Odoc_search.Res_type t -> (t.ty_name, RK_type)
+                 | Odoc_search.Res_exception e -> (e.ex_name, RK_exception)
+                 | Odoc_search.Res_attribute a -> (a.att_value.val_name, RK_attribute)
+                 | Odoc_search.Res_method m -> (m.met_value.val_name, RK_method)
+                 | Odoc_search.Res_section (_ ,t)-> assert false
+               in
+               add_verified (name, Some kind) ;
+               (name, Some kind)
+         in
+         match res with
+         | (name, Some k) -> Ref (name, Some k)
+         | (_, None) ->
+             match parent_name with
+               None ->
+                 Odoc_messages.pwarning (Odoc_messages.cross_element_not_found initial_name);
+                 Ref (initial_name, None)
+             | Some p ->
+                 let parent_name =
+                   match Name.father p with
+                     "" -> None
+                   | s -> Some s
+                 in
+                 iter_parent ?parent_name (Name.concat p initial_name)
+       in
+       iter_parent ~parent_name initial_name
       )
-  | Ref (name, Some kind) ->
+  | Ref (initial_name, Some kind) ->
       (
-       let v = (name, Some kind) in
-       if was_verified v then
-	 Ref (name, Some kind)
-       else
-	 match kind with
-	 | RK_section _ ->
-	     (
-	       (** we just verify that we find an element of this kind with this name *)
-	      try
-		let re = Str.regexp ("^"^(Str.quote name)^"$") in
-		let t = Odoc_search.find_section module_list re in
-		let v2 = (name, Some (RK_section t)) in
-		add_verified v2 ;
-		Ref (name, Some (RK_section t))
-	      with
-		Not_found ->
-		  Odoc_messages.pwarning (Odoc_messages.cross_section_not_found name);
-		  Ref (name, None)
-	     )
-	 | _ ->
-	     let (f,f_mes) =
-	       match kind with
-		 RK_module -> module_exists, Odoc_messages.cross_module_not_found
-	       | RK_module_type -> module_type_exists, Odoc_messages.cross_module_type_not_found
-	       | RK_class -> class_exists, Odoc_messages.cross_class_not_found
-	       | RK_class_type -> class_type_exists, Odoc_messages.cross_class_type_not_found
-	       | RK_value -> value_exists, Odoc_messages.cross_value_not_found
-	       | RK_type -> type_exists, Odoc_messages.cross_type_not_found
-	       | RK_exception -> exception_exists, Odoc_messages.cross_exception_not_found
-	       | RK_attribute -> attribute_exists, Odoc_messages.cross_attribute_not_found
-	       | RK_method -> method_exists, Odoc_messages.cross_method_not_found
-	       | RK_section _ -> assert false
-	     in
-	     if f name then
-	       (
-		add_verified v ;
-		Ref (name, Some kind)
-	       )
-	     else
-	       (
-		Odoc_messages.pwarning (f_mes name);
-		Ref (name, None)
-	       )
+       let rec iter_parent ?parent_name name =
+         let v = (name, Some kind) in
+         if was_verified v then
+           Ref (name, Some kind)
+         else
+           let res =
+             match kind with
+             | RK_section _ ->
+                 (
+                  (** we just verify that we find an element of this kind with this name *)
+                  try
+                    let re = Str.regexp ("^"^(Str.quote name)^"$") in
+                    let t = Odoc_search.find_section module_list re in
+                    let v2 = (name, Some (RK_section t)) in
+                    add_verified v2 ;
+                    (name, Some (RK_section t))
+                  with
+                    Not_found ->
+                      (name, None)
+                 )
+             | _ ->
+                 let f =
+                   match kind with
+                     RK_module -> module_exists
+                   | RK_module_type -> module_type_exists
+                   | RK_class -> class_exists
+                   | RK_class_type -> class_type_exists
+                   | RK_value -> value_exists
+                   | RK_type -> type_exists
+                   | RK_exception -> exception_exists
+                   | RK_attribute -> attribute_exists
+                   | RK_method -> method_exists
+                   | RK_section _ -> assert false
+                 in
+                 if f name then
+                   (
+                    add_verified v ;
+                    (name, Some kind)
+                   )
+                 else
+                   (name, None)
+           in
+           match res with
+           | (name, Some k) -> Ref (name, Some k)
+           | (_, None) ->
+               match parent_name with
+                 None ->
+                   Odoc_messages.pwarning (not_found_of_kind kind initial_name);
+                   Ref (initial_name, None)
+               | Some p ->
+                   let parent_name =
+                     match Name.father p with
+                       "" -> None
+                     | s -> Some s
+                   in
+                   iter_parent ?parent_name (Name.concat p initial_name)
+       in
+       iter_parent ~parent_name initial_name
       )
   | Module_list l ->
       Module_list l
   | Index_list ->
       Index_list
-  | Custom (s,t) -> Custom (s, (assoc_comments_text module_list t))
+  | Custom (s,t) -> Custom (s, (assoc_comments_text parent_name module_list t))
 
-and assoc_comments_text module_list text =
-  List.map (assoc_comments_text_elements module_list) text
+and assoc_comments_text parent_name module_list text =
+  List.map (assoc_comments_text_elements parent_name module_list) text
 
-and assoc_comments_info module_list i =
-  let ft = assoc_comments_text module_list in
+and assoc_comments_info parent_name module_list i =
+  let ft = assoc_comments_text parent_name module_list in
   {
     i with
     i_desc = ao ft i.i_desc ;
@@ -700,133 +747,155 @@ and assoc_comments_info module_list i =
   }
 
 
-let rec assoc_comments_module_element module_list m_ele =
+let rec assoc_comments_module_element parent_name module_list m_ele =
   match m_ele with
-    Element_module m -> Element_module (assoc_comments_module module_list m)
-  | Element_module_type mt -> Element_module_type (assoc_comments_module_type module_list mt)
-  | Element_included_module _ -> m_ele (* don't go down into the aliases *)
-  | Element_class c -> Element_class (assoc_comments_class module_list c)
-  | Element_class_type ct -> Element_class_type (assoc_comments_class_type module_list ct)
-  | Element_value v -> Element_value (assoc_comments_value module_list v)
-  | Element_exception e -> Element_exception (assoc_comments_exception module_list e)
-  | Element_type t -> Element_type (assoc_comments_type module_list t)
-  | Element_module_comment t -> Element_module_comment (assoc_comments_text module_list t)
+    Element_module m ->
+      Element_module (assoc_comments_module module_list m)
+  | Element_module_type mt ->
+      Element_module_type (assoc_comments_module_type module_list mt)
+  | Element_included_module _ ->
+      m_ele (* don't go down into the aliases *)
+  | Element_class c ->
+      Element_class (assoc_comments_class module_list c)
+  | Element_class_type ct ->
+      Element_class_type (assoc_comments_class_type module_list ct)
+  | Element_value v ->
+      Element_value (assoc_comments_value module_list v)
+  | Element_exception e ->
+      Element_exception (assoc_comments_exception module_list e)
+  | Element_type t ->
+      Element_type (assoc_comments_type module_list t)
+  | Element_module_comment t ->
+      Element_module_comment (assoc_comments_text parent_name module_list t)
 
-and assoc_comments_class_element module_list c_ele =
+and assoc_comments_class_element parent_name module_list c_ele =
   match c_ele with
-    Class_attribute a -> Class_attribute (assoc_comments_attribute module_list a)
-  | Class_method m -> Class_method (assoc_comments_method module_list m)
-  | Class_comment t -> Class_comment (assoc_comments_text module_list t)
+    Class_attribute a ->
+      Class_attribute (assoc_comments_attribute module_list a)
+  | Class_method m ->
+      Class_method (assoc_comments_method module_list m)
+  | Class_comment t ->
+      Class_comment (assoc_comments_text parent_name module_list t)
 
-and assoc_comments_module_kind module_list mk =
+and assoc_comments_module_kind parent_name module_list mk =
   match mk with
   | Module_struct eles ->
-      Module_struct (List.map (assoc_comments_module_element module_list) eles)
+      Module_struct
+        (List.map (assoc_comments_module_element parent_name module_list) eles)
   | Module_alias _
   | Module_functor _ ->
       mk
   | Module_apply (mk1, mk2) ->
-      Module_apply (assoc_comments_module_kind module_list mk1,
-                    assoc_comments_module_kind module_list mk2)
+      Module_apply (assoc_comments_module_kind parent_name module_list mk1,
+                    assoc_comments_module_kind parent_name module_list mk2)
   | Module_with (mtk, s) ->
-      Module_with (assoc_comments_module_type_kind module_list mtk, s)
+      Module_with (assoc_comments_module_type_kind parent_name module_list mtk, s)
   | Module_constraint (mk1, mtk) ->
-      Module_constraint (assoc_comments_module_kind module_list mk1,
-                         assoc_comments_module_type_kind module_list mtk)
+      Module_constraint
+        (assoc_comments_module_kind parent_name module_list mk1,
+         assoc_comments_module_type_kind parent_name module_list mtk)
 
-and assoc_comments_module_type_kind module_list mtk =
+and assoc_comments_module_type_kind parent_name module_list mtk =
   match mtk with
   | Module_type_struct eles ->
-      Module_type_struct (List.map (assoc_comments_module_element module_list) eles)
+      Module_type_struct
+        (List.map (assoc_comments_module_element parent_name module_list) eles)
   | Module_type_functor (params, mtk1) ->
-      Module_type_functor (params, assoc_comments_module_type_kind module_list mtk1)
+      Module_type_functor
+        (params, assoc_comments_module_type_kind parent_name module_list mtk1)
   | Module_type_alias _ ->
       mtk
   | Module_type_with (mtk1, s) ->
-      Module_type_with (assoc_comments_module_type_kind module_list mtk1, s)
+      Module_type_with
+        (assoc_comments_module_type_kind parent_name module_list mtk1, s)
 
-and assoc_comments_class_kind module_list ck =
+and assoc_comments_class_kind parent_name module_list ck =
   match ck with
     Class_structure (inher, eles) ->
       let inher2 =
         List.map
-          (fun ic -> { ic with
-                       ic_text = ao (assoc_comments_text module_list) ic.ic_text })
+          (fun ic ->
+            { ic with
+              ic_text = ao (assoc_comments_text parent_name module_list) ic.ic_text })
           inher
       in
-      Class_structure (inher2, List.map (assoc_comments_class_element module_list) eles)
+      Class_structure
+        (inher2, List.map (assoc_comments_class_element parent_name module_list) eles)
 
   | Class_apply _
   | Class_constr _ -> ck
   | Class_constraint (ck1, ctk) ->
-      Class_constraint (assoc_comments_class_kind module_list ck1,
-                        assoc_comments_class_type_kind module_list ctk)
+      Class_constraint (assoc_comments_class_kind parent_name module_list ck1,
+                        assoc_comments_class_type_kind parent_name module_list ctk)
 
-and assoc_comments_class_type_kind module_list ctk =
+and assoc_comments_class_type_kind parent_name module_list ctk =
   match ctk with
     Class_signature (inher, eles) ->
       let inher2 =
         List.map
           (fun ic -> { ic with
-                       ic_text = ao (assoc_comments_text module_list) ic.ic_text })
+                       ic_text = ao (assoc_comments_text parent_name module_list) ic.ic_text })
           inher
       in
-      Class_signature (inher2, List.map (assoc_comments_class_element module_list) eles)
+      Class_signature (inher2, List.map (assoc_comments_class_element parent_name module_list) eles)
 
   | Class_type _ -> ctk
 
 
 and assoc_comments_module module_list m =
-  m.m_info <- ao (assoc_comments_info module_list) m.m_info ;
-  m.m_kind <- assoc_comments_module_kind module_list m.m_kind ;
+  m.m_info <- ao (assoc_comments_info m.m_name module_list) m.m_info ;
+  m.m_kind <- assoc_comments_module_kind m.m_name module_list m.m_kind ;
   m
 
 and assoc_comments_module_type module_list mt =
-  mt.mt_info <- ao (assoc_comments_info module_list) mt.mt_info ;
-  mt.mt_kind <- ao (assoc_comments_module_type_kind module_list) mt.mt_kind ;
+  mt.mt_info <- ao (assoc_comments_info mt.mt_name module_list) mt.mt_info ;
+  mt.mt_kind <- ao (assoc_comments_module_type_kind mt.mt_name module_list) mt.mt_kind ;
   mt
 
 and assoc_comments_class module_list c =
-  c.cl_info <- ao (assoc_comments_info module_list) c.cl_info ;
-  c.cl_kind <- assoc_comments_class_kind module_list c.cl_kind ;
-  assoc_comments_parameter_list module_list c.cl_parameters;
+  c.cl_info <- ao (assoc_comments_info c.cl_name module_list) c.cl_info ;
+  c.cl_kind <- assoc_comments_class_kind c.cl_name module_list c.cl_kind ;
+  assoc_comments_parameter_list c.cl_name module_list c.cl_parameters;
   c
 
 and assoc_comments_class_type module_list ct =
-  ct.clt_info <- ao (assoc_comments_info module_list) ct.clt_info ;
-  ct.clt_kind <- assoc_comments_class_type_kind module_list ct.clt_kind ;
+  ct.clt_info <- ao (assoc_comments_info ct.clt_name module_list) ct.clt_info ;
+  ct.clt_kind <- assoc_comments_class_type_kind ct.clt_name module_list ct.clt_kind ;
   ct
 
-and assoc_comments_parameter module_list p =
+and assoc_comments_parameter parent_name module_list p =
   match p with
     Simple_name sn ->
-      sn.sn_text <- ao (assoc_comments_text module_list) sn.sn_text
+      sn.sn_text <- ao (assoc_comments_text parent_name module_list) sn.sn_text
   | Tuple (l, t) ->
-      List.iter (assoc_comments_parameter module_list) l
+      List.iter (assoc_comments_parameter parent_name module_list) l
 
-and assoc_comments_parameter_list module_list pl =
-  List.iter (assoc_comments_parameter module_list) pl
+and assoc_comments_parameter_list parent_name module_list pl =
+  List.iter (assoc_comments_parameter parent_name module_list) pl
 
 and assoc_comments_value module_list v =
-  v.val_info <- ao (assoc_comments_info module_list) v.val_info ;
-  assoc_comments_parameter_list module_list v.val_parameters;
+  let parent = Name.father v.val_name in
+  v.val_info <- ao (assoc_comments_info parent module_list) v.val_info ;
+  assoc_comments_parameter_list parent module_list v.val_parameters;
   v
 
 and assoc_comments_exception module_list e =
-  e.ex_info <- ao (assoc_comments_info module_list) e.ex_info ;
+  let parent = Name.father e.ex_name in
+  e.ex_info <- ao (assoc_comments_info parent module_list) e.ex_info ;
   e
 
 and assoc_comments_type module_list t =
-  t.ty_info <- ao (assoc_comments_info module_list) t.ty_info ;
+  let parent = Name.father t.ty_name in
+  t.ty_info <- ao (assoc_comments_info parent module_list) t.ty_info ;
   (match t.ty_kind with
     Type_abstract -> ()
   | Type_variant (vl, _) ->
       List.iter
-        (fun vc -> vc.vc_text <- ao (assoc_comments_text module_list) vc.vc_text)
+        (fun vc -> vc.vc_text <- ao (assoc_comments_text parent module_list) vc.vc_text)
         vl
   | Type_record (fl, _) ->
       List.iter
-        (fun rf -> rf.rf_text <- ao (assoc_comments_text module_list) rf.rf_text)
+        (fun rf -> rf.rf_text <- ao (assoc_comments_text parent module_list) rf.rf_text)
         fl
   );
   t
@@ -836,8 +905,9 @@ and assoc_comments_attribute module_list a =
   a
 
 and assoc_comments_method module_list m =
+  let parent_name = Name.father m.met_value.val_name in
   let _ = assoc_comments_value module_list m.met_value in
-  assoc_comments_parameter_list module_list m.met_value.val_parameters;
+  assoc_comments_parameter_list parent_name module_list m.met_value.val_parameters;
   m
 
 
@@ -897,6 +967,3 @@ let associate module_list =
 
   (* Find a type for each name of element which is referenced in comments. *)
   ignore (associate_type_of_elements_in_comments module_list)
-
-
-(* eof $Id: odoc_cross.ml,v 1.15.10.1 2005/11/07 15:59:04 doligez Exp $ *)
