@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: bigarray_stubs.c,v 1.19.4.1 2005/02/22 14:33:36 doligez Exp $ */
+/* $Id: bigarray_stubs.c,v 1.22 2006/01/27 14:33:42 doligez Exp $ */
 
 #include <stddef.h>
 #include <stdarg.h>
@@ -24,14 +24,14 @@
 #include "memory.h"
 #include "mlvalues.h"
 
-extern void bigarray_unmap_file(void * addr, unsigned long len);
+extern void caml_ba_unmap_file(void * addr, uintnat len);
                                           /* from mmap_xxx.c */
 
 /* Compute the number of elements of a big array */
 
-static unsigned long bigarray_num_elts(struct caml_bigarray * b)
+static uintnat caml_ba_num_elts(struct caml_bigarray * b)
 {
-  unsigned long num_elts;
+  uintnat num_elts;
   int i;
   num_elts = 1;
   for (i = 0; i < b->num_dims; i++) num_elts = num_elts * b->dim[i];
@@ -40,7 +40,7 @@ static unsigned long bigarray_num_elts(struct caml_bigarray * b)
 
 /* Size in bytes of a bigarray element, indexed by bigarray kind */
 
-int bigarray_element_size[] =
+int caml_ba_element_size[] =
 { 4 /*FLOAT32*/, 8 /*FLOAT64*/,
   1 /*SINT8*/, 1 /*UINT8*/,
   2 /*SINT16*/, 2 /*UINT16*/,
@@ -51,41 +51,42 @@ int bigarray_element_size[] =
 
 /* Compute the number of bytes for the elements of a big array */
 
-unsigned long bigarray_byte_size(struct caml_bigarray * b)
+uintnat caml_ba_byte_size(struct caml_bigarray * b)
 {
-  return bigarray_num_elts(b)
-         * bigarray_element_size[b->flags & BIGARRAY_KIND_MASK];
+  return caml_ba_num_elts(b)
+         * caml_ba_element_size[b->flags & CAML_BA_KIND_MASK];
 }
 
 /* Operation table for bigarrays */
 
-static void bigarray_finalize(value v);
-static int bigarray_compare(value v1, value v2);
-static long bigarray_hash(value v);
-static void bigarray_serialize(value, unsigned long *, unsigned long *);
-unsigned long bigarray_deserialize(void * dst);
-static struct custom_operations bigarray_ops = {
+static void caml_ba_finalize(value v);
+static int caml_ba_compare(value v1, value v2);
+static intnat caml_ba_hash(value v);
+static void caml_ba_serialize(value, uintnat *, uintnat *);
+uintnat caml_ba_deserialize(void * dst);
+static struct custom_operations caml_ba_ops = {
   "_bigarray",
-  bigarray_finalize,
-  bigarray_compare,
-  bigarray_hash,
-  bigarray_serialize,
-  bigarray_deserialize
+  caml_ba_finalize,
+  caml_ba_compare,
+  caml_ba_hash,
+  caml_ba_serialize,
+  caml_ba_deserialize
 };
 
 /* Multiplication of unsigned longs with overflow detection */
 
-static unsigned long
-bigarray_multov(unsigned long a, unsigned long b, int * overflow)
+static uintnat
+caml_ba_multov(uintnat a, uintnat b, int * overflow)
 {
-#define HALF_SIZE (sizeof(unsigned long) * 4)
-#define LOW_HALF(x) ((x) & ((1UL << HALF_SIZE) - 1))
+#define HALF_SIZE (sizeof(uintnat) * 4)
+#define HALF_MASK (((uintnat)1 << HALF_SIZE) - 1)
+#define LOW_HALF(x) ((x) & HALF_MASK)
 #define HIGH_HALF(x) ((x) >> HALF_SIZE)
   /* Cut in half words */
-  unsigned long al = LOW_HALF(a);
-  unsigned long ah = HIGH_HALF(a);
-  unsigned long bl = LOW_HALF(b);
-  unsigned long bh = HIGH_HALF(b);
+  uintnat al = LOW_HALF(a);
+  uintnat ah = HIGH_HALF(a);
+  uintnat bl = LOW_HALF(b);
+  uintnat bh = HIGH_HALF(b);
   /* Exact product is:
               al * bl
            +  ah * bl  << HALF_SIZE
@@ -98,11 +99,11 @@ bigarray_multov(unsigned long a, unsigned long b, int * overflow)
      OR the sum al * bl + LOW_HALF(ah * bl) << HALF_SIZE
                         + LOW_HALF(al * bh) << HALF_SIZE overflows.
      This sum is equal to p = (a * b) modulo word size. */
-  unsigned long p1 = al * bh;
-  unsigned long p2 = ah * bl;
-  unsigned long p = a * b;
+  uintnat p1 = al * bh;
+  uintnat p2 = ah * bl;
+  uintnat p = a * b;
   if (ah != 0 && bh != 0) *overflow = 1;
-  if (p1 >= (1UL << HALF_SIZE) || p2 >= (1UL << HALF_SIZE)) *overflow = 1;
+  if (HIGH_HALF(p1) != 0 || HIGH_HALF(p2) != 0) *overflow = 1;
   p1 <<= HALF_SIZE;
   p2 <<= HALF_SIZE;
   p1 += p2;
@@ -115,24 +116,24 @@ bigarray_multov(unsigned long a, unsigned long b, int * overflow)
 
 /* Allocation of a big array */
 
-#define MAX_BIGARRAY_MEMORY 256*1024*1024
+#define CAML_BA_MAX_MEMORY 256*1024*1024
 /* 256 Mb -- after allocating that much, it's probably worth speeding
    up the major GC */
 
-/* [alloc_bigarray] will allocate a new bigarray object in the heap.
+/* [caml_ba_alloc] will allocate a new bigarray object in the heap.
    If [data] is NULL, the memory for the contents is also allocated
-   (with [malloc]) by [alloc_bigarray].
+   (with [malloc]) by [caml_ba_alloc].
    [data] cannot point into the Caml heap.
    [dim] may point into an object in the Caml heap.
 */
 CAMLexport value
-alloc_bigarray(int flags, int num_dims, void * data, long * dim)
+caml_ba_alloc(int flags, int num_dims, void * data, intnat * dim)
 {
-  unsigned long num_elts, size;
+  uintnat num_elts, size;
   int overflow, i;
   value res;
   struct caml_bigarray * b;
-  long dimcopy[MAX_NUM_DIMS];
+  intnat dimcopy[MAX_NUM_DIMS];
 
   Assert(num_dims >= 1 && num_dims <= MAX_NUM_DIMS);
   Assert((flags & BIGARRAY_KIND_MASK) <= BIGARRAY_COMPLEX64);
@@ -142,20 +143,20 @@ alloc_bigarray(int flags, int num_dims, void * data, long * dim)
     overflow = 0;
     num_elts = 1;
     for (i = 0; i < num_dims; i++) {
-      num_elts = bigarray_multov(num_elts, dimcopy[i], &overflow);
+      num_elts = caml_ba_multov(num_elts, dimcopy[i], &overflow);
     }
-    size = bigarray_multov(num_elts, 
-                           bigarray_element_size[flags & BIGARRAY_KIND_MASK],
-                           &overflow);
+    size = caml_ba_multov(num_elts,
+                          caml_ba_element_size[flags & BIGARRAY_KIND_MASK],
+                          &overflow);
     if (overflow) raise_out_of_memory();
     data = malloc(size);
     if (data == NULL && size != 0) raise_out_of_memory();
     flags |= BIGARRAY_MANAGED;
   }
-  res = alloc_custom(&bigarray_ops,
-                     sizeof(struct caml_bigarray) 
-                     + (num_dims - 1) * sizeof(long),
-                     size, MAX_BIGARRAY_MEMORY);
+  res = alloc_custom(&caml_ba_ops,
+                     sizeof(struct caml_ba_array)
+                     + (num_dims - 1) * sizeof(intnat),
+                     size, CAML_BA_MAX_MEMORY);
   b = Bigarray_val(res);
   b->data = data;
   b->num_dims = num_dims;
@@ -165,28 +166,28 @@ alloc_bigarray(int flags, int num_dims, void * data, long * dim)
   return res;
 }
 
-/* Same as alloc_bigarray, but dimensions are passed as a list of
+/* Same as caml_ba_alloc, but dimensions are passed as a list of
    arguments */
 
-CAMLexport value alloc_bigarray_dims(int flags, int num_dims, void * data, ...)
+CAMLexport value caml_ba_alloc_dims(int flags, int num_dims, void * data, ...)
 {
   va_list ap;
-  long dim[MAX_NUM_DIMS];
+  intnat dim[MAX_NUM_DIMS];
   int i;
   value res;
 
   va_start(ap, data);
-  for (i = 0; i < num_dims; i++) dim[i] = va_arg(ap, long);
+  for (i = 0; i < num_dims; i++) dim[i] = va_arg(ap, intnat);
   va_end(ap);
-  res = alloc_bigarray(flags, num_dims, data, dim);
+  res = caml_ba_alloc(flags, num_dims, data, dim);
   return res;
 }
 
 /* Allocate a bigarray from Caml */
 
-CAMLprim value bigarray_create(value vkind, value vlayout, value vdim)
+CAMLprim value caml_ba_create(value vkind, value vlayout, value vdim)
 {
-  long dim[MAX_NUM_DIMS];
+  intnat dim[MAX_NUM_DIMS];
   mlsize_t num_dims;
   int i, flags;
 
@@ -199,30 +200,30 @@ CAMLprim value bigarray_create(value vkind, value vlayout, value vdim)
       invalid_argument("Bigarray.create: negative dimension");
   }
   flags = Int_val(vkind) | Int_val(vlayout);
-  return alloc_bigarray(flags, num_dims, NULL, dim);
+  return caml_ba_alloc(flags, num_dims, NULL, dim);
 }
 
 /* Given a big array and a vector of indices, check that the indices
    are within the bounds and return the offset of the corresponding
    array element in the data part of the array. */
 
-static long bigarray_offset(struct caml_bigarray * b, long * index)
+static long caml_ba_offset(struct caml_ba_array * b, intnat * index)
 {
-  long offset;
+  intnat offset;
   int i;
 
   offset = 0;
   if ((b->flags & BIGARRAY_LAYOUT_MASK) == BIGARRAY_C_LAYOUT) {
     /* C-style layout: row major, indices start at 0 */
     for (i = 0; i < b->num_dims; i++) {
-      if ((unsigned long) index[i] >= (unsigned long) b->dim[i])
+      if ((uintnat) index[i] >= (uintnat) b->dim[i])
         array_bound_error();
       offset = offset * b->dim[i] + index[i];
     }
   } else {
     /* Fortran-style layout: column major, indices start at 1 */
     for (i = b->num_dims - 1; i >= 0; i--) {
-      if ((unsigned long) (index[i] - 1) >= (unsigned long) b->dim[i])
+      if ((uintnat) (index[i] - 1) >= (uintnat) b->dim[i])
         array_bound_error();
       offset = offset * b->dim[i] + (index[i] - 1);
     }
@@ -242,12 +243,12 @@ static value copy_two_doubles(double d0, double d1)
 
 /* Generic code to read from a big array */
 
-value bigarray_get_N(value vb, value * vind, int nind)
+value caml_ba_get_N(value vb, value * vind, int nind)
 {
   struct caml_bigarray * b = Bigarray_val(vb);
-  long index[MAX_NUM_DIMS];
+  intnat index[MAX_NUM_DIMS];
   int i;
-  long offset;
+  intnat offset;
 
   /* Check number of indices = number of dimensions of array
      (maybe not necessary if ML typing guarantees this) */
@@ -255,7 +256,7 @@ value bigarray_get_N(value vb, value * vind, int nind)
     invalid_argument("Bigarray.get: wrong number of indices");
   /* Compute offset and check bounds */
   for (i = 0; i < b->num_dims; i++) index[i] = Long_val(vind[i]);
-  offset = bigarray_offset(b, index);
+  offset = caml_ba_offset(b, index);
   /* Perform read */
   switch ((b->flags) & BIGARRAY_KIND_MASK) {
   default:
@@ -265,9 +266,9 @@ value bigarray_get_N(value vb, value * vind, int nind)
   case BIGARRAY_FLOAT64:
     return copy_double(((double *) b->data)[offset]);
   case BIGARRAY_SINT8:
-    return Val_int(((schar *) b->data)[offset]);
+    return Val_int(((int8 *) b->data)[offset]);
   case BIGARRAY_UINT8:
-    return Val_int(((unsigned char *) b->data)[offset]);
+    return Val_int(((uint8 *) b->data)[offset]);
   case BIGARRAY_SINT16:
     return Val_int(((int16 *) b->data)[offset]);
   case BIGARRAY_UINT16:
@@ -277,9 +278,9 @@ value bigarray_get_N(value vb, value * vind, int nind)
   case BIGARRAY_INT64:
     return copy_int64(((int64 *) b->data)[offset]);
   case BIGARRAY_NATIVE_INT:
-    return copy_nativeint(((long *) b->data)[offset]);
+    return copy_nativeint(((intnat *) b->data)[offset]);
   case BIGARRAY_CAML_INT:
-    return Val_long(((long *) b->data)[offset]);
+    return Val_long(((intnat *) b->data)[offset]);
   case BIGARRAY_COMPLEX32:
     { float * p = ((float *) b->data) + offset * 2;
       return copy_two_doubles(p[0], p[1]); }
@@ -289,74 +290,74 @@ value bigarray_get_N(value vb, value * vind, int nind)
   }
 }
 
-CAMLprim value bigarray_get_1(value vb, value vind1)
+CAMLprim value caml_ba_get_1(value vb, value vind1)
 {
-  return bigarray_get_N(vb, &vind1, 1);
+  return caml_ba_get_N(vb, &vind1, 1);
 }
 
-CAMLprim value bigarray_get_2(value vb, value vind1, value vind2)
+CAMLprim value caml_ba_get_2(value vb, value vind1, value vind2)
 {
   value vind[2];
   vind[0] = vind1; vind[1] = vind2;
-  return bigarray_get_N(vb, vind, 2);
+  return caml_ba_get_N(vb, vind, 2);
 }
 
-CAMLprim value bigarray_get_3(value vb, value vind1, value vind2, value vind3)
+CAMLprim value caml_ba_get_3(value vb, value vind1, value vind2, value vind3)
 {
   value vind[3];
   vind[0] = vind1; vind[1] = vind2; vind[2] = vind3;
-  return bigarray_get_N(vb, vind, 3);
+  return caml_ba_get_N(vb, vind, 3);
 }
 
 #if 0
-CAMLprim value bigarray_get_4(value vb, value vind1, value vind2, 
+CAMLprim value caml_ba_get_4(value vb, value vind1, value vind2,
                      value vind3, value vind4)
 {
   value vind[4];
   vind[0] = vind1; vind[1] = vind2; vind[2] = vind3; vind[3] = vind4;
-  return bigarray_get_N(vb, vind, 4);
+  return caml_ba_get_N(vb, vind, 4);
 }
 
-CAMLprim value bigarray_get_5(value vb, value vind1, value vind2, 
+CAMLprim value caml_ba_get_5(value vb, value vind1, value vind2,
                      value vind3, value vind4, value vind5)
 {
   value vind[5];
-  vind[0] = vind1; vind[1] = vind2; vind[2] = vind3; 
+  vind[0] = vind1; vind[1] = vind2; vind[2] = vind3;
   vind[3] = vind4; vind[4] = vind5;
-  return bigarray_get_N(vb, vind, 5);
+  return caml_ba_get_N(vb, vind, 5);
 }
 
-CAMLprim value bigarray_get_6(value vb, value vind1, value vind2, 
+CAMLprim value caml_ba_get_6(value vb, value vind1, value vind2,
                      value vind3, value vind4, value vind5, value vind6)
 {
   value vind[6];
-  vind[0] = vind1; vind[1] = vind2; vind[2] = vind3; 
+  vind[0] = vind1; vind[1] = vind2; vind[2] = vind3;
   vind[3] = vind4; vind[4] = vind5; vind[5] = vind6;
-  return bigarray_get_N(vb, vind, 6);
+  return caml_ba_get_N(vb, vind, 6);
 }
 #endif
 
-CAMLprim value bigarray_get_generic(value vb, value vind)
+CAMLprim value caml_ba_get_generic(value vb, value vind)
 {
-  return bigarray_get_N(vb, &Field(vind, 0), Wosize_val(vind));
+  return caml_ba_get_N(vb, &Field(vind, 0), Wosize_val(vind));
 }
 
 /* Generic write to a big array */
 
-static value bigarray_set_aux(value vb, value * vind, long nind, value newval)
+static value caml_ba_set_aux(value vb, value * vind, intnat nind, value newval)
 {
   struct caml_bigarray * b = Bigarray_val(vb);
-  long index[MAX_NUM_DIMS];
+  intnat index[MAX_NUM_DIMS];
   int i;
-  long offset;
-  
+  intnat offset;
+
   /* Check number of indices = number of dimensions of array
      (maybe not necessary if ML typing guarantees this) */
   if (nind != b->num_dims)
     invalid_argument("Bigarray.set: wrong number of indices");
   /* Compute offset and check bounds */
   for (i = 0; i < b->num_dims; i++) index[i] = Long_val(vind[i]);
-  offset = bigarray_offset(b, index);
+  offset = caml_ba_offset(b, index);
   /* Perform write */
   switch (b->flags & BIGARRAY_KIND_MASK) {
   default:
@@ -367,7 +368,7 @@ static value bigarray_set_aux(value vb, value * vind, long nind, value newval)
     ((double *) b->data)[offset] = Double_val(newval); break;
   case BIGARRAY_SINT8:
   case BIGARRAY_UINT8:
-    ((schar *) b->data)[offset] = Int_val(newval); break;
+    ((int8 *) b->data)[offset] = Int_val(newval); break;
   case BIGARRAY_SINT16:
   case BIGARRAY_UINT16:
     ((int16 *) b->data)[offset] = Int_val(newval); break;
@@ -376,9 +377,9 @@ static value bigarray_set_aux(value vb, value * vind, long nind, value newval)
   case BIGARRAY_INT64:
     ((int64 *) b->data)[offset] = Int64_val(newval); break;
   case BIGARRAY_NATIVE_INT:
-    ((long *) b->data)[offset] = Nativeint_val(newval); break;
+    ((intnat *) b->data)[offset] = Nativeint_val(newval); break;
   case BIGARRAY_CAML_INT:
-    ((long *) b->data)[offset] = Long_val(newval); break;
+    ((intnat *) b->data)[offset] = Long_val(newval); break;
   case BIGARRAY_COMPLEX32:
     { float * p = ((float *) b->data) + offset * 2;
       p[0] = Double_field(newval, 0);
@@ -393,68 +394,68 @@ static value bigarray_set_aux(value vb, value * vind, long nind, value newval)
   return Val_unit;
 }
 
-CAMLprim value bigarray_set_1(value vb, value vind1, value newval)
+CAMLprim value caml_ba_set_1(value vb, value vind1, value newval)
 {
-  return bigarray_set_aux(vb, &vind1, 1, newval);
+  return caml_ba_set_aux(vb, &vind1, 1, newval);
 }
 
-CAMLprim value bigarray_set_2(value vb, value vind1, value vind2, value newval)
+CAMLprim value caml_ba_set_2(value vb, value vind1, value vind2, value newval)
 {
   value vind[2];
   vind[0] = vind1; vind[1] = vind2;
-  return bigarray_set_aux(vb, vind, 2, newval);
+  return caml_ba_set_aux(vb, vind, 2, newval);
 }
 
-CAMLprim value bigarray_set_3(value vb, value vind1, value vind2, value vind3,
+CAMLprim value caml_ba_set_3(value vb, value vind1, value vind2, value vind3,
                      value newval)
 {
   value vind[3];
   vind[0] = vind1; vind[1] = vind2; vind[2] = vind3;
-  return bigarray_set_aux(vb, vind, 3, newval);
+  return caml_ba_set_aux(vb, vind, 3, newval);
 }
 
 #if 0
-CAMLprim value bigarray_set_4(value vb, value vind1, value vind2, 
+CAMLprim value caml_ba_set_4(value vb, value vind1, value vind2,
                      value vind3, value vind4, value newval)
 {
   value vind[4];
   vind[0] = vind1; vind[1] = vind2; vind[2] = vind3; vind[3] = vind4;
-  return bigarray_set_aux(vb, vind, 4, newval);
+  return caml_ba_set_aux(vb, vind, 4, newval);
 }
 
-CAMLprim value bigarray_set_5(value vb, value vind1, value vind2, 
+CAMLprim value caml_ba_set_5(value vb, value vind1, value vind2,
                      value vind3, value vind4, value vind5, value newval)
 {
   value vind[5];
-  vind[0] = vind1; vind[1] = vind2; vind[2] = vind3; 
+  vind[0] = vind1; vind[1] = vind2; vind[2] = vind3;
   vind[3] = vind4; vind[4] = vind5;
-  return bigarray_set_aux(vb, vind, 5, newval);
+  return caml_ba_set_aux(vb, vind, 5, newval);
 }
 
-CAMLprim value bigarray_set_6(value vb, value vind1, value vind2, 
+CAMLprim value caml_ba_set_6(value vb, value vind1, value vind2,
                      value vind3, value vind4, value vind5,
                      value vind6, value newval)
 {
   value vind[6];
-  vind[0] = vind1; vind[1] = vind2; vind[2] = vind3; 
+  vind[0] = vind1; vind[1] = vind2; vind[2] = vind3;
   vind[3] = vind4; vind[4] = vind5; vind[5] = vind6;
-  return bigarray_set_aux(vb, vind, 6, newval);
+  return caml_ba_set_aux(vb, vind, 6, newval);
 }
 
-value bigarray_set_N(value vb, value * vind, int nargs)
+value caml_ba_set_N(value vb, value * vind, int nargs)
 {
-  return bigarray_set_aux(vb, vind, nargs - 1, vind[nargs - 1]);
+  return caml_ba_set_aux(vb, vind, nargs - 1, vind[nargs - 1]);
 }
 #endif
 
-CAMLprim value bigarray_set_generic(value vb, value vind, value newval)
+CAMLprim value caml_ba_set_generic(value vb, value vind, value newval)
 {
-  return bigarray_set_aux(vb, &Field(vind, 0), Wosize_val(vind), newval);
+  return caml_ba_set_aux(vb, &Field(vind, 0), Wosize_val(vind), newval);
 }
 
 /* Return the number of dimensions of a big array */
 
-CAMLprim value bigarray_num_dims(value vb)
+CAMLprim value caml_ba_num_dims(value vb)
 {
   struct caml_bigarray * b = Bigarray_val(vb);
   return Val_long(b->num_dims);
@@ -462,31 +463,31 @@ CAMLprim value bigarray_num_dims(value vb)
 
 /* Return the n-th dimension of a big array */
 
-CAMLprim value bigarray_dim(value vb, value vn)
+CAMLprim value caml_ba_dim(value vb, value vn)
 {
   struct caml_bigarray * b = Bigarray_val(vb);
-  long n = Long_val(vn);
+  intnat n = Long_val(vn);
   if (n >= b->num_dims) invalid_argument("Bigarray.dim");
   return Val_long(b->dim[n]);
 }
 
 /* Return the kind of a big array */
 
-CAMLprim value bigarray_kind(value vb)
+CAMLprim value caml_ba_kind(value vb)
 {
   return Val_int(Bigarray_val(vb)->flags & BIGARRAY_KIND_MASK);
 }
 
 /* Return the layout of a big array */
 
-CAMLprim value bigarray_layout(value vb)
+CAMLprim value caml_ba_layout(value vb)
 {
   return Val_int(Bigarray_val(vb)->flags & BIGARRAY_LAYOUT_MASK);
 }
 
 /* Finalization of a big array */
 
-static void bigarray_finalize(value v)
+static void caml_ba_finalize(value v)
 {
   struct caml_bigarray * b = Bigarray_val(v);
 
@@ -505,10 +506,10 @@ static void bigarray_finalize(value v)
     break;
   case BIGARRAY_MAPPED_FILE:
     if (b->proxy == NULL) {
-      bigarray_unmap_file(b->data, bigarray_byte_size(b));
+      caml_ba_unmap_file(b->data, caml_ba_byte_size(b));
     } else {
       if (-- b->proxy->refcount == 0) {
-        bigarray_unmap_file(b->proxy->data, b->proxy->size);
+        caml_ba_unmap_file(b->proxy->data, b->proxy->size);
         stat_free(b->proxy);
       }
     }
@@ -518,23 +519,23 @@ static void bigarray_finalize(value v)
 
 /* Comparison of two big arrays */
 
-static int bigarray_compare(value v1, value v2)
+static int caml_ba_compare(value v1, value v2)
 {
   struct caml_bigarray * b1 = Bigarray_val(v1);
   struct caml_bigarray * b2 = Bigarray_val(v2);
-  unsigned long n, num_elts;
+  uintnat n, num_elts;
   int i;
 
   /* Compare number of dimensions */
   if (b1->num_dims != b2->num_dims) return b2->num_dims - b1->num_dims;
   /* Same number of dimensions: compare dimensions lexicographically */
   for (i = 0; i < b1->num_dims; i++) {
-    long d1 = b1->dim[i];
-    long d2 = b2->dim[i];
+    intnat d1 = b1->dim[i];
+    intnat d2 = b2->dim[i];
     if (d1 != d2) return d1 < d2 ? -1 : 1;
   }
   /* Same dimensions: compare contents lexicographically */
-  num_elts = bigarray_num_elts(b1);
+  num_elts = caml_ba_num_elts(b1);
 
 #define DO_INTEGER_COMPARISON(type) \
   { type * p1 = b1->data; type * p2 = b2->data; \
@@ -570,9 +571,9 @@ static int bigarray_compare(value v1, value v2)
   case BIGARRAY_FLOAT64:
     DO_FLOAT_COMPARISON(double);
   case BIGARRAY_SINT8:
-    DO_INTEGER_COMPARISON(schar);
+    DO_INTEGER_COMPARISON(int8);
   case BIGARRAY_UINT8:
-    DO_INTEGER_COMPARISON(unsigned char);
+    DO_INTEGER_COMPARISON(uint8);
   case BIGARRAY_SINT16:
     DO_INTEGER_COMPARISON(int16);
   case BIGARRAY_UINT16:
@@ -596,7 +597,7 @@ static int bigarray_compare(value v1, value v2)
 #endif
   case BIGARRAY_CAML_INT:
   case BIGARRAY_NATIVE_INT:
-    DO_INTEGER_COMPARISON(long);
+    DO_INTEGER_COMPARISON(intnat);
   default:
     Assert(0);
     return 0;                   /* should not happen */
@@ -607,10 +608,10 @@ static int bigarray_compare(value v1, value v2)
 
 /* Hashing of a bigarray */
 
-static long bigarray_hash(value v)
+static intnat caml_ba_hash(value v)
 {
   struct caml_bigarray * b = Bigarray_val(v);
-  long num_elts, n, h;
+  intnat num_elts, n, h;
   int i;
 
   num_elts = 1;
@@ -623,13 +624,13 @@ static long bigarray_hash(value v)
   switch (b->flags & BIGARRAY_KIND_MASK) {
   case BIGARRAY_SINT8:
   case BIGARRAY_UINT8: {
-    unsigned char * p = b->data;
+    uint8 * p = b->data;
     for (n = 0; n < num_elts; n++) h = COMBINE(h, *p++);
     break;
   }
   case BIGARRAY_SINT16:
   case BIGARRAY_UINT16: {
-    unsigned short * p = b->data;
+    uint16 * p = b->data;
     for (n = 0; n < num_elts; n++) h = COMBINE(h, *p++);
     break;
   }
@@ -654,7 +655,7 @@ static long bigarray_hash(value v)
 #endif
 #ifdef ARCH_SIXTYFOUR
   {
-    unsigned long * p = b->data;
+    uintnat * p = b->data;
     for (n = 0; n < num_elts; n++) h = COMBINE(h, *p++);
     break;
   }
@@ -676,13 +677,13 @@ static long bigarray_hash(value v)
   return h;
 }
 
-static void bigarray_serialize_longarray(void * data,
-                                         long num_elts,
-                                         long min_val, long max_val)
+static void caml_ba_serialize_longarray(void * data,
+                                        intnat num_elts,
+                                        intnat min_val, intnat max_val)
 {
 #ifdef ARCH_SIXTYFOUR
   int overflow_32 = 0;
-  long * p, n;
+  intnat * p, n;
   for (n = 0, p = data; n < num_elts; n++, p++) {
     if (*p < min_val || *p > max_val) { overflow_32 = 1; break; }
   }
@@ -699,12 +700,12 @@ static void bigarray_serialize_longarray(void * data,
 #endif
 }
 
-static void bigarray_serialize(value v, 
-                               unsigned long * wsize_32,
-                               unsigned long * wsize_64)
+static void caml_ba_serialize(value v,
+                              uintnat * wsize_32,
+                              uintnat * wsize_64)
 {
   struct caml_bigarray * b = Bigarray_val(v);
-  long num_elts;
+  intnat num_elts;
   int i;
 
   /* Serialize header information */
@@ -733,10 +734,10 @@ static void bigarray_serialize(value v,
   case BIGARRAY_COMPLEX64:
     serialize_block_8(b->data, num_elts * 2); break;
   case BIGARRAY_CAML_INT:
-    bigarray_serialize_longarray(b->data, num_elts, -0x40000000, 0x3FFFFFFF);
+    caml_ba_serialize_longarray(b->data, num_elts, -0x40000000, 0x3FFFFFFF);
     break;
   case BIGARRAY_NATIVE_INT:
-    bigarray_serialize_longarray(b->data, num_elts, -0x80000000, 0x7FFFFFFF);
+    caml_ba_serialize_longarray(b->data, num_elts, -0x80000000, 0x7FFFFFFF);
     break;
   }
   /* Compute required size in Caml heap.  Assumes struct caml_bigarray
@@ -746,14 +747,14 @@ static void bigarray_serialize(value v,
   *wsize_64 = (4 + b->num_dims) * 8;
 }
 
-static void bigarray_deserialize_longarray(void * dest, long num_elts)
+static void caml_ba_deserialize_longarray(void * dest, intnat num_elts)
 {
   int sixty = deserialize_uint_1();
 #ifdef ARCH_SIXTYFOUR
   if (sixty) {
     deserialize_block_8(dest, num_elts);
   } else {
-    long * p, n;
+    intnat * p, n;
     for (n = 0, p = dest; n < num_elts; n++, p++) *p = deserialize_sint_4();
   }
 #else
@@ -764,11 +765,11 @@ static void bigarray_deserialize_longarray(void * dest, long num_elts)
 #endif
 }
 
-unsigned long bigarray_deserialize(void * dst)
+uintnat caml_ba_deserialize(void * dst)
 {
   struct caml_bigarray * b = dst;
   int i, elt_size;
-  unsigned long num_elts;
+  uintnat num_elts;
 
   /* Read back header information */
   b->num_dims = deserialize_uint_4();
@@ -776,11 +777,11 @@ unsigned long bigarray_deserialize(void * dst)
   b->proxy = NULL;
   for (i = 0; i < b->num_dims; i++) b->dim[i] = deserialize_uint_4();
   /* Compute total number of elements */
-  num_elts = bigarray_num_elts(b);
+  num_elts = caml_ba_num_elts(b);
   /* Determine element size in bytes */
   if ((b->flags & BIGARRAY_KIND_MASK) > BIGARRAY_COMPLEX64)
     deserialize_error("input_value: bad bigarray kind");
-  elt_size = bigarray_element_size[b->flags & BIGARRAY_KIND_MASK];
+  elt_size = caml_ba_element_size[b->flags & BIGARRAY_KIND_MASK];
   /* Allocate room for data */
   b->data = malloc(elt_size * num_elts);
   if (b->data == NULL)
@@ -805,15 +806,15 @@ unsigned long bigarray_deserialize(void * dst)
     deserialize_block_8(b->data, num_elts * 2); break;
   case BIGARRAY_CAML_INT:
   case BIGARRAY_NATIVE_INT:
-    bigarray_deserialize_longarray(b->data, num_elts); break;
+    caml_ba_deserialize_longarray(b->data, num_elts); break;
   }
-  return sizeof(struct caml_bigarray) + (b->num_dims - 1) * sizeof(long);
+  return sizeof(struct caml_bigarray) + (b->num_dims - 1) * sizeof(intnat);
 }
 
 /* Create / update proxy to indicate that b2 is a sub-array of b1 */
 
-static void bigarray_update_proxy(struct caml_bigarray * b1,
-                                  struct caml_bigarray * b2)
+static void caml_ba_update_proxy(struct caml_bigarray * b1,
+                                 struct caml_bigarray * b2)
 {
   struct caml_bigarray_proxy * proxy;
   /* Nothing to do for un-managed arrays */
@@ -829,7 +830,7 @@ static void bigarray_update_proxy(struct caml_bigarray * b1,
     proxy->refcount = 2;      /* original array + sub array */
     proxy->data = b1->data;
     proxy->size =
-      b1->flags & BIGARRAY_MAPPED_FILE ? bigarray_byte_size(b1) : 0;
+      b1->flags & BIGARRAY_MAPPED_FILE ? caml_ba_byte_size(b1) : 0;
     b1->proxy = proxy;
     b2->proxy = proxy;
   }
@@ -837,15 +838,15 @@ static void bigarray_update_proxy(struct caml_bigarray * b1,
 
 /* Slicing */
 
-CAMLprim value bigarray_slice(value vb, value vind)
+CAMLprim value caml_ba_slice(value vb, value vind)
 {
   CAMLparam2 (vb, vind);
   #define b ((struct caml_bigarray *) Bigarray_val(vb))
   CAMLlocal1 (res);
-  long index[MAX_NUM_DIMS];
+  intnat index[MAX_NUM_DIMS];
   int num_inds, i;
-  long offset;
-  long * sub_dims;
+  intnat offset;
+  intnat * sub_dims;
   char * sub_data;
 
   /* Check number of indices < number of dimensions of array */
@@ -857,23 +858,23 @@ CAMLprim value bigarray_slice(value vb, value vind)
     /* We slice from the left */
     for (i = 0; i < num_inds; i++) index[i] = Long_val(Field(vind, i));
     for (/*nothing*/; i < b->num_dims; i++) index[i] = 0;
-    offset = bigarray_offset(b, index);
+    offset = caml_ba_offset(b, index);
     sub_dims = b->dim + num_inds;
   } else {
     /* We slice from the right */
     for (i = 0; i < num_inds; i++)
       index[b->num_dims - num_inds + i] = Long_val(Field(vind, i));
     for (i = 0; i < b->num_dims - num_inds; i++) index[i] = 1;
-    offset = bigarray_offset(b, index);
+    offset = caml_ba_offset(b, index);
     sub_dims = b->dim;
   }
   sub_data =
     (char *) b->data +
-    offset * bigarray_element_size[b->flags & BIGARRAY_KIND_MASK];
+    offset * caml_ba_element_size[b->flags & BIGARRAY_KIND_MASK];
   /* Allocate a Caml bigarray to hold the result */
   res = alloc_bigarray(b->flags, b->num_dims - num_inds, sub_data, sub_dims);
   /* Create or update proxy in case of managed bigarray */
-  bigarray_update_proxy(b, Bigarray_val(res));
+  caml_ba_update_proxy(b, Bigarray_val(res));
   /* Return result */
   CAMLreturn (res);
 
@@ -882,15 +883,15 @@ CAMLprim value bigarray_slice(value vb, value vind)
 
 /* Extracting a sub-array of same number of dimensions */
 
-CAMLprim value bigarray_sub(value vb, value vofs, value vlen)
+CAMLprim value caml_ba_sub(value vb, value vofs, value vlen)
 {
   CAMLparam3 (vb, vofs, vlen);
   CAMLlocal1 (res);
   #define b ((struct caml_bigarray *) Bigarray_val(vb))
-  long ofs = Long_val(vofs);
-  long len = Long_val(vlen);
+  intnat ofs = Long_val(vofs);
+  intnat len = Long_val(vlen);
   int i, changed_dim;
-  long mul;
+  intnat mul;
   char * sub_data;
 
   /* Compute offset and check bounds */
@@ -910,13 +911,13 @@ CAMLprim value bigarray_sub(value vb, value vofs, value vlen)
     invalid_argument("Bigarray.sub: bad sub-array");
   sub_data =
     (char *) b->data +
-    ofs * mul * bigarray_element_size[b->flags & BIGARRAY_KIND_MASK];
+    ofs * mul * caml_ba_element_size[b->flags & BIGARRAY_KIND_MASK];
   /* Allocate a Caml bigarray to hold the result */
   res = alloc_bigarray(b->flags, b->num_dims, sub_data, b->dim);
   /* Doctor the changed dimension */
   Bigarray_val(res)->dim[changed_dim] = len;
   /* Create or update proxy in case of managed bigarray */
-  bigarray_update_proxy(b, Bigarray_val(res));
+  caml_ba_update_proxy(b, Bigarray_val(res));
   /* Return result */
   CAMLreturn (res);
 
@@ -925,12 +926,12 @@ CAMLprim value bigarray_sub(value vb, value vofs, value vlen)
 
 /* Copying a big array into another one */
 
-CAMLprim value bigarray_blit(value vsrc, value vdst)
+CAMLprim value caml_ba_blit(value vsrc, value vdst)
 {
   struct caml_bigarray * src = Bigarray_val(vsrc);
   struct caml_bigarray * dst = Bigarray_val(vdst);
   int i;
-  long num_bytes;
+  intnat num_bytes;
 
   /* Check same numbers of dimensions and same dimensions */
   if (src->num_dims != dst->num_dims) goto blit_error;
@@ -938,8 +939,8 @@ CAMLprim value bigarray_blit(value vsrc, value vdst)
     if (src->dim[i] != dst->dim[i]) goto blit_error;
   /* Compute number of bytes in array data */
   num_bytes =
-    bigarray_num_elts(src)
-    * bigarray_element_size[src->flags & BIGARRAY_KIND_MASK];
+    caml_ba_num_elts(src)
+    * caml_ba_element_size[src->flags & BIGARRAY_KIND_MASK];
   /* Do the copying */
   memmove (dst->data, src->data, num_bytes);
   return Val_unit;
@@ -950,10 +951,10 @@ CAMLprim value bigarray_blit(value vsrc, value vdst)
 
 /* Filling a big array with a given value */
 
-CAMLprim value bigarray_fill(value vb, value vinit)
+CAMLprim value caml_ba_fill(value vb, value vinit)
 {
   struct caml_bigarray * b = Bigarray_val(vb);
-  long num_elts = bigarray_num_elts(b);
+  intnat num_elts = caml_ba_num_elts(b);
 
   switch (b->flags & BIGARRAY_KIND_MASK) {
   default:
@@ -977,10 +978,10 @@ CAMLprim value bigarray_fill(value vb, value vinit)
     for (p = b->data; num_elts > 0; p++, num_elts--) *p = init;
     break;
   }
-  case BIGARRAY_SINT16: 
+  case BIGARRAY_SINT16:
   case BIGARRAY_UINT16: {
     int init = Int_val(vinit);
-    short * p;
+    int16 * p;
     for (p = b->data; num_elts > 0; p++, num_elts--) *p = init;
     break;
   }
@@ -997,14 +998,14 @@ CAMLprim value bigarray_fill(value vb, value vinit)
     break;
   }
   case BIGARRAY_NATIVE_INT: {
-    long init = Nativeint_val(vinit);
-    long * p;
+    intnat init = Nativeint_val(vinit);
+    intnat * p;
     for (p = b->data; num_elts > 0; p++, num_elts--) *p = init;
     break;
   }
   case BIGARRAY_CAML_INT: {
-    long init = Long_val(vinit);
-    long * p;
+    intnat init = Long_val(vinit);
+    intnat * p;
     for (p = b->data; num_elts > 0; p++, num_elts--) *p = init;
     break;
   }
@@ -1029,14 +1030,14 @@ CAMLprim value bigarray_fill(value vb, value vinit)
 /* Reshape an array: change dimensions and number of dimensions, preserving
    array contents */
 
-CAMLprim value bigarray_reshape(value vb, value vdim)
+CAMLprim value caml_ba_reshape(value vb, value vdim)
 {
   CAMLparam2 (vb, vdim);
   CAMLlocal1 (res);
   #define b ((struct caml_bigarray *) Bigarray_val(vb))
-  long dim[MAX_NUM_DIMS];
+  intnat dim[MAX_NUM_DIMS];
   mlsize_t num_dims;
-  unsigned long num_elts;
+  uintnat num_elts;
   int i;
 
   num_dims = Wosize_val(vdim);
@@ -1050,12 +1051,12 @@ CAMLprim value bigarray_reshape(value vb, value vdim)
     num_elts *= dim[i];
   }
   /* Check that sizes agree */
-  if (num_elts != bigarray_num_elts(b))
+  if (num_elts != caml_ba_num_elts(b))
     invalid_argument("Bigarray.reshape: size mismatch");
   /* Create bigarray with same data and new dimensions */
   res = alloc_bigarray(b->flags, num_dims, b->data, dim);
   /* Create or update proxy in case of managed bigarray */
-  bigarray_update_proxy(b, Bigarray_val(res));
+  caml_ba_update_proxy(b, Bigarray_val(res));
   /* Return result */
   CAMLreturn (res);
 
@@ -1064,8 +1065,8 @@ CAMLprim value bigarray_reshape(value vb, value vdim)
 
 /* Initialization */
 
-CAMLprim value bigarray_init(value unit)
+CAMLprim value caml_ba_init(value unit)
 {
-  register_custom_operations(&bigarray_ops);
+  register_custom_operations(&caml_ba_ops);
   return Val_unit;
 }
