@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../../LICENSE.  *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: extract_crc.ml,v 1.8 2001/06/18 09:17:59 ddr Exp $ *)
+(* $Id: extract_crc.ml,v 1.10 2004/04/09 13:26:41 xleroy Exp $ *)
 
 (* Print the digests of unit interfaces *)
 
@@ -32,7 +33,9 @@ let print_crc unit =
     prerr_endline unit;
     begin match exn with
       Sys_error msg -> prerr_endline msg
-    | Dynlink.Error _ -> prerr_endline "Ill formed .cmi file"
+    | Dynlink.Error(Dynlink.File_not_found name) ->
+        prerr_string "Cannot find file "; prerr_endline name
+    | Dynlink.Error _ -> prerr_endline "Ill-formed .cmi file"
     | _ -> raise exn
     end;
     exit 2
