@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: image.h,v 1.6 2000/02/10 14:04:58 xleroy Exp $ */
+/* $Id: image.h,v 1.8 2004/03/24 15:02:06 starynke Exp $ */
 
 struct grimage {
   int width, height;            /* Dimensions of the image */
@@ -25,4 +26,4 @@ struct grimage {
 
 #define Transparent (-1)
 
-value gr_new_image(int w, int h);
+value caml_gr_new_image(int w, int h);
