@@ -102,7 +102,6 @@ FileReadGIF(interp, f, fileName, formatString)
     char newresbuf[640];
     char *imageName;
     char *resultptr;
-    int prevpos;
     int loop = -1;
 
     if((winPtr = Tk_MainWindow(interp)) == NULL){
@@ -335,8 +334,11 @@ FileReadGIF(interp, f, fileName, formatString)
                 goto error;
             }
         }
-        Tk_PhotoPutBlock(photoHandle, &block, 0, 0, 
-                         imageWidth, imageHeight);
+        Tk_PhotoPutBlock(photoHandle, &block, 0, 0, imageWidth, imageHeight
+#if (TK_MAJOR_VERSION == 8 && TK_MINOR_VERSION >= 4 || TK_MAJOR_VERSION > 8)
+                   , TK_PHOTO_COMPOSITE_SET
+#endif
+          );
 #ifdef TKANIM_DEBUG
     fprintf(stderr, " Retrieving result\n");
 #endif
