@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: posix.c,v 1.53.2.1 2006/03/22 13:13:45 xleroy Exp $ */
+/* $Id: posix.c,v 1.55 2007/01/29 12:11:17 xleroy Exp $ */
 
 /* Thread interface for POSIX 1003.1c threads */
 
@@ -88,10 +88,10 @@ struct caml_thread_struct {
   value * trapsp;               /* Saved value of trapsp for this thread */
   struct caml__roots_block * local_roots; /* Saved value of local_roots */
   struct longjmp_buffer * external_raise; /* Saved external_raise */
+#endif
   int backtrace_pos;            /* Saved backtrace_pos */
   code_t * backtrace_buffer;    /* Saved backtrace_buffer */
   value backtrace_last_exn;     /* Saved backtrace_last_exn (root) */
-#endif
 };
 
 typedef struct caml_thread_struct * caml_thread_t;
@@ -147,9 +147,7 @@ static void caml_thread_scan_roots(scanning_action action)
   th = curr_thread;
   do {
     (*action)(th->descr, &th->descr);
-#ifndef NATIVE_CODE
     (*action)(th->backtrace_last_exn, &th->backtrace_last_exn);
-#endif
     /* Don't rescan the stack of the current thread, it was done already */
     if (th != curr_thread) {
 #ifdef NATIVE_CODE
@@ -186,10 +184,10 @@ static void caml_thread_enter_blocking_section(void)
   curr_thread->trapsp = trapsp;
   curr_thread->local_roots = local_roots;
   curr_thread->external_raise = external_raise;
+#endif
   curr_thread->backtrace_pos = backtrace_pos;
   curr_thread->backtrace_buffer = backtrace_buffer;
   curr_thread->backtrace_last_exn = backtrace_last_exn;
-#endif
   /* Tell other threads that the runtime is free */
   pthread_mutex_lock(&caml_runtime_mutex);
   caml_runtime_busy = 0;
@@ -226,10 +224,10 @@ static void caml_thread_leave_blocking_section(void)
   trapsp = curr_thread->trapsp;
   local_roots = curr_thread->local_roots;
   external_raise = curr_thread->external_raise;
+#endif
   backtrace_pos = curr_thread->backtrace_pos;
   backtrace_buffer = curr_thread->backtrace_buffer;
   backtrace_last_exn = curr_thread->backtrace_last_exn;
-#endif
 }
 
 static int caml_thread_try_leave_blocking_section(void)
@@ -409,8 +407,8 @@ static void caml_thread_stop(void)
 #ifndef NATIVE_CODE
   /* Free the memory resources */
   stat_free(th->stack_low);
-  if (th->backtrace_buffer != NULL) free(th->backtrace_buffer);
 #endif
+  if (th->backtrace_buffer != NULL) free(th->backtrace_buffer);
   /* Free the thread descriptor */
   stat_free(th);
 }
@@ -479,10 +477,10 @@ value caml_thread_new(value clos)          /* ML */
     th->trapsp = th->stack_high;
     th->local_roots = NULL;
     th->external_raise = NULL;
+#endif
     th->backtrace_pos = 0;
     th->backtrace_buffer = NULL;
     th->backtrace_last_exn = Val_unit;
-#endif
     /* Add thread info block to the list of threads */
     th->next = curr_thread->next;
     th->prev = curr_thread;
@@ -529,9 +527,7 @@ value caml_thread_uncaught_exception(value exn)  /* ML */
   fprintf(stderr, "Thread %d killed on uncaught exception %s\n",
           Int_val(Ident(curr_thread->descr)), msg);
   free(msg);
-#ifndef NATIVE_CODE
-  if (backtrace_active) print_exception_backtrace();
-#endif
+  if (caml_backtrace_active) print_exception_backtrace();
   fflush(stderr);
   return Val_unit;
 }
