@@ -6,18 +6,19 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: umask.c,v 1.7 1999/11/17 18:58:09 xleroy Exp $ */
+/* $Id: umask.c,v 1.9 2001/12/07 13:40:37 xleroy Exp $ */
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <mlvalues.h>
 #include "unixsupport.h"
 
-value unix_umask(value perm)           /* ML */
+CAMLprim value unix_umask(value perm)
 {
   return Val_int(umask(Int_val(perm)));
 }
