@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: unix.mli,v 1.81.4.2 2006/03/21 15:50:29 doligez Exp $ *)
+(* $Id: unix.mli,v 1.85 2007/01/22 07:41:00 garrigue Exp $ *)
 
 (** Interface to the Unix system *)
 
@@ -339,16 +339,19 @@ type stats =
 (** The informations returned by the {!Unix.stat} calls. *)
 
 val stat : string -> stats
-(** Return the informations for the named file. *)
+(** Return the information for the named file. *)
 
 val lstat : string -> stats
 (** Same as {!Unix.stat}, but in case the file is a symbolic link,
-   return the informations for the link itself. *)
+   return the information for the link itself. *)
 
 val fstat : file_descr -> stats
-(** Return the informations for the file associated with the given
+(** Return the information for the file associated with the given
    descriptor. *)
 
+val isatty : file_descr -> bool
+(** Return [true] if the given file descriptor refers to a terminal or
+   console window, [false] otherwise. *)
 
 (** {6 File operations on large files} *)
 
