@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: close_on.c,v 1.6 1999/11/17 18:58:13 xleroy Exp $ */
+/* $Id: close_on.c,v 1.8 2001/12/07 13:40:43 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <windows.h>
@@ -24,7 +25,7 @@ int win_set_inherit(value fd, BOOL inherit)
   if (! DuplicateHandle(GetCurrentProcess(), oldh,
                         GetCurrentProcess(), &newh,
                         0L, inherit, DUPLICATE_SAME_ACCESS)) {
-    _dosmaperr(GetLastError());
+    win32_maperr(GetLastError());
     return -1;
   }
   Handle_val(fd) = newh;
@@ -32,13 +33,13 @@ int win_set_inherit(value fd, BOOL inherit)
   return 0;
 }
 
-value win_set_close_on_exec(value fd)             /* ML */
+CAMLprim value win_set_close_on_exec(value fd)
 {
   if (win_set_inherit(fd, FALSE) == -1) uerror("set_close_on_exec", Nothing);
   return Val_unit;
 }
 
-value win_clear_close_on_exec(value fd)             /* ML */
+CAMLprim value win_clear_close_on_exec(value fd)
 {
   if (win_set_inherit(fd, TRUE) == -1) uerror("clear_close_on_exec", Nothing);
   return Val_unit;
