@@ -6,19 +6,18 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: createprocess.c,v 1.9 2000/02/13 17:27:46 xleroy Exp $ */
+/* $Id: createprocess.c,v 1.13 2001/12/07 13:40:43 xleroy Exp $ */
 
 #include <windows.h>
 #include <mlvalues.h>
+#include <osdeps.h>
 #include "unixsupport.h"
 
-/* From the Caml runtime */
-extern char * searchpath(char * name);
-
 static int win_has_console(void);
 
 value win_create_process_native(value cmd, value cmdline, value env,
@@ -29,8 +28,7 @@ value win_create_process_native(value cmd, value cmdline, value env,
   char * exefile, * envp;
   int flags;
 
-  exefile = searchpath(String_val(cmd));
-  if (exefile == NULL) exefile = String_val(cmd);
+  exefile = search_exe_in_path(String_val(cmd));
   if (env != Val_int(0)) {
     envp = String_val(Field(env, 0));
   } else {
@@ -42,19 +40,23 @@ value win_create_process_native(value cmd, value cmdline, value env,
   si.hStdInput = Handle_val(fd1);
   si.hStdOutput = Handle_val(fd2);
   si.hStdError = Handle_val(fd3);
-  /* If we do not have a console window, then we must run
-     console mode applications as detached processes.
-     Otherwise, a new console is created and the redirections
-     are ignored.  If we're running a GUI application, the
-     detached / non-detached flag doesn't matter. */
+  /* If we do not have a console window, then we must create one
+     before running the process (keep it hidden for apparence).
+     Also one must suppress spurious flags in si.dwFlags.
+     Otherwise the redirections are ignored.
+     If we are starting a GUI application, the newly created
+     console should not matter. */
   if (win_has_console())
     flags = 0;
-  else
-    flags = DETACHED_PROCESS;
+  else {
+    flags = CREATE_NEW_CONSOLE;
+    si.dwFlags = (STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES);
+    si.wShowWindow = SW_HIDE;
+  }
   /* Create the process */
   if (! CreateProcess(exefile, String_val(cmdline), NULL, NULL,
                       TRUE, flags, envp, NULL, &si, &pi)) {
-    _dosmaperr(GetLastError());
+    win32_maperr(GetLastError());
     uerror("create_process", cmd);
   }
   CloseHandle(pi.hThread);
@@ -63,7 +65,7 @@ value win_create_process_native(value cmd, value cmdline, value env,
   return Val_int(pi.hProcess);
 }
 
-value win_create_process(value * argv, int argn) /* ML */
+CAMLprim value win_create_process(value * argv, int argn)
 {
   return win_create_process_native(argv[0], argv[1], argv[2],
                                    argv[3], argv[4], argv[5]);
@@ -71,7 +73,8 @@ value win_create_process(value * argv, int argn) /* ML */
 
 static int win_has_console(void)
 {
-  HANDLE h;
+  HANDLE h, log;
+  int i;
 
   h = CreateFile("CONOUT$", GENERIC_WRITE, FILE_SHARE_WRITE, NULL,
                  OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
