@@ -11,12 +11,11 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: dup2.c,v 1.7 2003/01/06 14:52:57 xleroy Exp $ */
+/* $Id: dup2.c,v 1.9 2006/09/21 09:43:58 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
-extern value win_fd_handle(value);
 extern int _dup2(int, int);
 
 CAMLprim value unix_dup2(value fd1, value fd2)
@@ -38,6 +37,6 @@ CAMLprim value unix_dup2(value fd1, value fd2)
   Descr_kind_val(fd2) = Descr_kind_val(fd1);
   /* Reflect the dup2 on the CRT fds, if any */
   if (CRT_fd_val(fd1) != NO_CRT_FD || CRT_fd_val(fd2) != NO_CRT_FD)
-    _dup2(Int_val(win_fd_handle(fd1)), Int_val(win_fd_handle(fd2)));
+    _dup2(win_CRT_fd_of_filedescr(fd1), win_CRT_fd_of_filedescr(fd2));
   return Val_unit;
 }
