@@ -11,16 +11,15 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: listen.c,v 1.8 2001/12/07 13:40:45 xleroy Exp $ */
+/* $Id: listen.c,v 1.9 2002/04/30 15:00:46 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
-#include <winsock.h>
 
 CAMLprim value unix_listen(sock, backlog)
      value sock, backlog;
 {
-  if (listen((SOCKET) Handle_val(sock), Int_val(backlog)) == -1) {
+  if (listen(Socket_val(sock), Int_val(backlog)) == -1) {
     win32_maperr(WSAGetLastError());
     uerror("listen", Nothing);
   }
