@@ -11,12 +11,10 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: socket.c,v 1.11 2001/12/07 13:40:46 xleroy Exp $ */
+/* $Id: socket.c,v 1.12 2002/04/30 15:00:47 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
-#include <sys/types.h>
-#include <winsock.h>
 
 int socket_domain_table[] = {
   PF_UNIX, PF_INET
@@ -53,5 +51,5 @@ CAMLprim value unix_socket(domain, type, proto)
     win32_maperr(WSAGetLastError());
     uerror("socket", Nothing);
   }
-  return win_alloc_handle((HANDLE) s);
+  return win_alloc_socket(s);
 }
