@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: winwait.c,v 1.15 2005/09/22 14:21:50 xleroy Exp $ */
+/* $Id: winwait.c,v 1.18 2006/10/18 08:26:54 xleroy Exp $ */
 
 #include <windows.h>
 #include <mlvalues.h>
@@ -41,13 +41,18 @@ static int wait_flag_table[] = { CAML_WNOHANG, CAML_WUNTRACED };
 CAMLprim value win_waitpid(value vflags, value vpid_req)
 {
   int flags;
-  DWORD status;
+  DWORD status, retcode;
   HANDLE pid_req = (HANDLE) Long_val(vpid_req);
+  DWORD err = 0;
 
   flags = convert_flag_list(vflags, wait_flag_table);
   if ((flags & CAML_WNOHANG) == 0) {
-    if (WaitForSingleObject(pid_req, INFINITE) == WAIT_FAILED) {
-      win32_maperr(GetLastError());
+    enter_blocking_section();
+    retcode = WaitForSingleObject(pid_req, INFINITE);
+    if (retcode == WAIT_FAILED) err = GetLastError();
+    leave_blocking_section();
+    if (err) {
+      win32_maperr(err);
       uerror("waitpid", Nothing);
     }
   }
