@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: printf.mli,v 1.46 2005/09/26 10:12:01 weis Exp $ *)
+(* $Id: printf.mli,v 1.54 2006/11/17 08:34:05 weis Exp $ *)
 
 (** Formatted output functions. *)
 
@@ -62,7 +62,7 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
      the format specified by the second letter.
    - [Ld], [Li], [Lu], [Lx], [LX], [Lo]: convert an [int64] argument to
      the format specified by the second letter.
-   - [a]: user-defined printer. Takes two arguments and apply the
+   - [a]: user-defined printer. Takes two arguments and applies the
      first one to [outchan] (the current output channel) and to the
      second argument. The first argument must therefore have type
      [out_channel -> 'b -> unit] and the second ['b].
@@ -72,7 +72,7 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
      [out_channel -> unit]) and apply it to [outchan].
    - [\{ fmt %\}]: convert a format string argument. The argument must
      have the same type as the internal format string [fmt].
-   - [\( fmt %\)]: format string substitution. Takes a format string
+   - [( fmt %)]: format string substitution. Takes a format string
      argument and substitutes it to the internal format string [fmt]
      to print following arguments. The argument must have the same
      type as [fmt].
@@ -113,16 +113,20 @@ val printf : ('a, out_channel, unit) format -> 'a
 val eprintf : ('a, out_channel, unit) format -> 'a
 (** Same as {!Printf.fprintf}, but output on [stderr]. *)
 
+val ifprintf : 'a -> ('b, 'a, unit) format -> 'b
+(** Same as {!Printf.fprintf}, but does not print anything.
+    Useful to ignore some material when conditionally printing. *)
+
 val sprintf : ('a, unit, string) format -> 'a
 (** Same as {!Printf.fprintf}, but instead of printing on an output channel,
-   return a string containing the result of formatting
-   the arguments. *)
+   return a string containing the result of formatting the arguments. *)
 
 val bprintf : Buffer.t -> ('a, Buffer.t, unit) format -> 'a
 (** Same as {!Printf.fprintf}, but instead of printing on an output channel,
    append the formatted arguments to the given extensible buffer
    (see module {!Buffer}). *)
 
+(** Formatted output functions with continuations. *)
 val kfprintf : (out_channel -> 'a) -> out_channel ->
               ('b, out_channel, unit, 'a) format4 -> 'b;;
 (** Same as [fprintf], but instead of returning immediately,
@@ -132,25 +136,73 @@ val ksprintf : (string -> 'a) -> ('b, unit, string, 'a) format4 -> 'b;;
 (** Same as [sprintf] above, but instead of returning the string,
    passes it to the first argument. *)
 
+val kbprintf : (Buffer.t -> 'a) -> Buffer.t ->
+              ('b, Buffer.t, unit, 'a) format4 -> 'b;;
+(** Same as [bprintf], but instead of returning immediately,
+   passes the buffer to its first argument at the end of printing. *)
+
 val kprintf : (string -> 'a) -> ('b, unit, string, 'a) format4 -> 'b;;
 (** A deprecated synonym for [ksprintf]. *)
 
 (**/**)
 
 (* For system use only.  Don't call directly. *)
-type index;;
 
-external index_of_int : int -> index = "%identity";;
+module CamlinternalPr : sig
 
-val scan_format : string -> 'a array -> index -> int ->
-  (index -> string -> int -> 'b) ->
-  (index -> 'c -> 'd -> int -> 'b) ->
-  (index -> 'e -> int -> 'b) ->
-  (index -> int -> 'b) ->
-  (index -> ('h, 'i, 'j, 'k) format4 -> int -> 'b) -> 'b
+  module Sformat : sig
+    type index;;
+
+    val index_of_int : int -> index;;
+    external int_of_index : index -> int = "%identity";;
+    external unsafe_index_of_int : int -> index = "%identity";;
+
+    val succ_index : index -> index;;
+
+    val sub : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> index -> int -> string;;
+    val to_string : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> string;;
+    external length : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> int
+      = "%string_length";;
+    external get : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> int -> char
+      = "%string_safe_get";;
+    external unsafe_to_string : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> string
+      = "%identity";;
+    external unsafe_get : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> int -> char
+      = "%string_unsafe_get";;
+
+  end;;
+
+  module Tformat : sig
+
+    type ac = {
+      mutable ac_rglr : int;
+      mutable ac_skip : int;
+      mutable ac_rdrs : int;
+    };;
+
+    val ac_of_format : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> ac;;
+
+    val sub_format :
+        (('a, 'b, 'c, 'd, 'e, 'f) format6 -> int) ->
+        (('a, 'b, 'c, 'd, 'e, 'f) format6 -> int -> char -> int) ->
+        char -> ('a, 'b, 'c, 'd, 'e, 'f) format6 -> int -> int
+
+    val summarize_format_type : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> string
+
+    val scan_format : ('a, 'b, 'c, 'd, 'e, 'f) format6 ->
+        'g array ->
+        Sformat.index ->
+        int ->
+        (Sformat.index -> string -> int -> 'h) ->
+        (Sformat.index -> 'i -> 'j -> int -> 'h) ->
+        (Sformat.index -> 'k -> int -> 'h) ->
+        (Sformat.index -> int -> 'h) ->
+        (Sformat.index -> ('l, 'm, 'n, 'o, 'p, 'q) format6 -> int -> 'h) -> 'h
+
+    val kapr :
+        (('a, 'b, 'c, 'd, 'e, 'f) format6 -> Obj.t array -> 'g) ->
+        ('a, 'b, 'c, 'd, 'e, 'f) format6 -> 'g
+  end;;
+
+end;;
 
-val sub_format :
-  (string -> int) -> (string -> int -> char -> int) ->
-  char -> string -> int -> int
-val summarize_format_type : string -> string
-val kapr : (string -> Obj.t array -> 'a) -> string -> 'a
