@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: weak.mli,v 1.13 2002/01/23 17:54:11 doligez Exp $ *)
+(* $Id: weak.mli,v 1.15 2004/02/02 14:43:12 doligez Exp $ *)
 
 (** Arrays of weak pointers and hash tables of weak pointers. *)
 
@@ -23,7 +23,11 @@ type 'a t
    pointer is a value that the garbage collector may erase at
    any time.
    A weak pointer is said to be full if it points to a value,
-   empty if the value was erased by the GC.*)
+   empty if the value was erased by the GC.
+   Note that weak arrays cannot be marshaled using
+   {!Pervasives.output_value} or the functions of the {!Marshal}
+   module.
+*)
 
 
 val create : int -> 'a t
@@ -91,14 +95,17 @@ val blit : 'a t -> int -> 'a t -> int -> int -> unit
     is [true].
 
     The [equal] relation must be able to work on a shallow copy of
-    the values and give the same result as with the value itself.
+    the values and give the same result as with the values themselves.
     *)
 
 module type S = sig
   type data
     (** The type of the elements stored in the table. *)
   type t
-    (** The type of tables that contain elements of type [data]. *)
+    (** The type of tables that contain elements of type [data].
+        Note that weak hash tables cannot be marshaled using
+        {!Pervasives.output_value} or the functions of the {!Marshal}
+        module. *)
   val create : int -> t
     (** [create n] creates a new empty weak hash table, of initial
         size [n].  The table will grow as needed. *)
