@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: dumpapprox.ml,v 1.14 2002/07/22 08:22:51 xleroy Exp $ *)
+(* $Id: dumpapprox.ml,v 1.15 2003/03/06 16:00:16 xleroy Exp $ *)
 
 (* Dump a .cmx file *)
 
@@ -40,7 +40,7 @@ let rec print_approx ppf = function
           if i > 0 then printf ";@ ";
           printf "%i: %a" i print_approx approx.(i)
         done in
-      printf "@[<hov 1>%a@]" tuple approx
+      printf "@[<hov 1>(%a)@]" tuple approx
   | Value_unknown ->
       print_string "_"
   | Value_integer n ->
