@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: topmain.ml,v 1.34 2003/07/25 12:18:25 xleroy Exp $ *)
+(* $Id: topmain.ml,v 1.39 2005/01/28 17:52:58 doligez Exp $ *)
 
 open Clflags
 
@@ -45,20 +45,29 @@ let file_argument name =
       else exit 2
     end
 
+let print_version () =
+  Printf.printf "The Objective Caml toplevel, version %s\n" Sys.ocaml_version;
+  exit 0;
+;;
+
 let main () =
   Arg.parse [
      "-I", Arg.String(fun dir ->
        let dir = Misc.expand_directory Config.standard_library dir in
        include_dirs := dir :: !include_dirs),
            "<dir>  Add <dir> to the list of include directories";
+     "-init", Arg.String (fun s -> init_file := Some s),
+           "<file>  Load <file> instead of default init file";
      "-labels", Arg.Clear classic, " Labels commute (default)";
      "-noassert", Arg.Set noassert, " Do not compile assertion checks";
      "-nolabels", Arg.Set classic, " Ignore labels and do not commute";
+     "-noprompt", Arg.Set noprompt, " Suppress all prompts";
      "-nostdlib", Arg.Set no_std_include,
            " do not add default directory to the list of include directories";
      "-principal", Arg.Set principal, " Check principality of type inference";
      "-rectypes", Arg.Set recursive_types, " Allow arbitrary recursive types";
      "-unsafe", Arg.Set fast, " No bound checking on array and string access";
+     "-version", Arg.Unit print_version, " Print version and exit";
      "-w", Arg.String (Warnings.parse_options false),
            "<flags>  Enable or disable warnings according to <flags>:\n\
        \032    A/a enable/disable all warnings\n\
@@ -72,11 +81,12 @@ let main () =
        \032    S/s enable/disable non-unit statement\n\
        \032    U/u enable/disable unused match case\n\
        \032    V/v enable/disable hidden instance variable\n\
+       \032    Y/y enable/disable suspicious unused variables\n\
+       \032    Z/z enable/disable all other unused variables\n\
        \032    X/x enable/disable all other warnings\n\
-       \032    default setting is \"Ale\"\n\
-       \032    (all warnings but labels and fragile match enabled)";
+       \032    default setting is \"Aelz\"";
      "-warn-error" , Arg.String (Warnings.parse_options true),
-       "<flags>  Enable or disable fatal warnings according to <flags>\n\
+       "<flags>  Treat the warnings of <flags> as errors, if they are enabled.\n\
          \032    (see option -w for the list of flags)\n\
          \032    default setting is a (all warnings are non-fatal)";
 
