@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typedecl.mli,v 1.29 2005/08/13 20:59:37 doligez Exp $ *)
+(* $Id: typedecl.mli,v 1.30 2006/11/02 01:10:04 garrigue Exp $ *)
 
 (* Typing of type definitions and primitive definitions *)
 
@@ -30,7 +30,8 @@ val transl_value_decl:
     Env.t -> Parsetree.value_description -> value_description
 
 val transl_with_constraint:
-    Env.t -> Path.t option -> Parsetree.type_declaration -> type_declaration
+    Env.t -> Ident.t -> Path.t option ->
+    Parsetree.type_declaration -> type_declaration
 
 val abstract_type_decl: int -> type_declaration
 val approx_type_decl:
