@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: types.ml,v 1.24 2004/06/12 08:55:49 xleroy Exp $ *)
+(* $Id: types.ml,v 1.26 2006/04/05 02:28:13 garrigue Exp $ *)
 
 (* Representation of types and declarations *)
 
@@ -90,7 +90,8 @@ and value_kind =
   | Val_prim of Primitive.description   (* Primitive *)
   | Val_ivar of mutable_flag * string   (* Instance variable (mutable ?) *)
   | Val_self of (Ident.t * type_expr) Meths.t ref *
-                (Ident.t * Asttypes.mutable_flag * type_expr) Vars.t ref *
+                (Ident.t * Asttypes.mutable_flag *
+                 Asttypes.virtual_flag * type_expr) Vars.t ref *
                 string * type_expr
                                         (* Self *)
   | Val_anc of (string * Ident.t) list * string
@@ -156,7 +157,8 @@ type class_type =
 
 and class_signature =
   { cty_self: type_expr;
-    cty_vars: (Asttypes.mutable_flag * type_expr) Vars.t;
+    cty_vars:
+      (Asttypes.mutable_flag * Asttypes.virtual_flag * type_expr) Vars.t;
     cty_concr: Concr.t;
     cty_inher: (Path.t * type_expr list) list }
 
@@ -164,12 +166,14 @@ type class_declaration =
   { cty_params: type_expr list;
     mutable cty_type: class_type;
     cty_path: Path.t;
-    cty_new: type_expr option }
+    cty_new: type_expr option;
+    cty_variance: (bool * bool) list }
 
 type cltype_declaration =
   { clty_params: type_expr list;
     clty_type: class_type;
-    clty_path: Path.t }
+    clty_path: Path.t;
+    clty_variance: (bool * bool) list }
 
 (* Type expressions for the module language *)
 
